'Programm ......... : TWI-Master fr eine LED-Ansteuerung (LAD)
'
'Programminhalt ... : - Beispielprogramm fr eine Masterbaugruppe (TWI)
'                       + TWI-Buskopplung einer LED-Kette
'                       + Steuerung einer LCD-Anzeige
'                       + Zeitschaltuhr ber DCF77-Empfang
'                       + Statusanzeige ber LED
'                       + Ermittlung aller angeschlossenen Slave-Adressen
'                         Die maximale Anzahl der in dieser Version anschlie-
'                         baren Slaves betrgt 40 und ist bei der Dimensio-
'                         nierung des Arrays <Slv_wert(40)> vernderbar.
'
'           optional: - Anbindung einer SDcard (Beispielcode am Ende)
'
'Programmierer .... : Ingolf Bauer (ingolf.bauer@nexgo.de)
'Version .......... : V 2.0.13
'Datum ............ : 07.08.2018
'
'Hinweise
'Busbezeichnung    "TWI" und "I2C" sind Bezeichnungen fr ein technisch identi-
'                  sches Bussystem.
'
'Label A_...       Diese Labels dienen nur dem schnelleren Auffinden des Pro-
'                  grammcodes.
'
'M04-Automatik     Auf dem LCD kann aus Platzgrnden nicht angezeigt werden,
'                  dass mit den Tasten FT1/3 eine Fortschaltung/Unterbrechung
'                  bei einigen Programmen nach oder in einem Zyklusdurchlauf
'                  (z. B. LED-Kette) mglich ist.
'                  Der Programmwechsel erfolgt alle 30 s. Diese Einstellung kann
'                  verndert werden (Label: A_450_updcf).
'
'-------------------------------------------------------------------------------
'
'Simulator-Einstellmglichkeit
'-----------------------------
'$sim                                              'Wartezeit aus
'
' Achtung!
' Diese Zeile ist unbedingt auszukommentieren, bevor der AVR programmiert wird,
' da sonst der AVR nicht korrekt arbeitet.
' Die Aktivierung ist nur fr den Simulator vorgesehen.
'
'-------------------------------------------------------------------------------
'
'Information
'-----------
'- Programmstart
'  Nach einer neuen Programmierung ist es unbedingt erforderlich, dass alle am
'  TWI-Bus angeschlossenen AVR abgeschaltet und wieder neu eingeschaltet werden.
'
'- Programmierung Drehschalter (DRS) - Stand 10/2016
'  Der BASCOM-Befehl "encoder" ist im Programm nicht verwendbar, da der gesamte
'  Port als "Input" initialisiert sein muss. Auerdem arbeitet der Befehl nicht
'  fehlerfrei.
'  Daher erfolgt die Auswertung des DRS auf eine andere Art und Weise.
'
'-------------------------------------------------------------------------------
'00. === I N F O R M A T I O N =================================================
'-------------------------------------------------------------------------------
'
A_010_bedienung:
'Bedienung
'---------
'Start
'-FT1 ....... Anzeige der Slave-ADR der angeschlossenen Busteilnehmer
'
'Programm
'-HGB ein
' FT1 ....... Programmfortschaltung + ..... Programmanzeige per LCD
' FT2 ....... Programmfortschaltung - ..... Programmanzeige per LCD
' FT3 ....... Programmunterbrechung ....... Programmanzeige per LCD

' FT1 ....... LCD ......................... Einstellung HGB-Kontrast
' FT1 ....... LCD ......................... Einstellung HGB-Helligkeit

'-HGB aus
' FT1-FT3 ... LCD ....................... HGB einschalten

'-DCF (bei Empfangsstrung; das Datum kann manuell nicht aktualisiert werden)
' FT1 ....... Eingabe der Stunden .... Einzelimpuls/Dauerbetrieb
' FT2 ....... Eingabe der Minuten .... Einzelimpuls/Dauerbetrieb
' FT3 ....... Sekunden setzen (00) ... Einzelimpuls

'-DRS (Modus 00-10)
' 01 ........ M00-PG: xx ................ Programmwahl manuell
' 02 ........ Funktuhrmodus (Empfang: bl) Zeit  xx.xx.xx - Datum
' 03 ........ M02-Wert .................. LCD-Kontrast
' 04 ........ M03-Wert .................. LCD-HGB (Hintergrundbeleuchtung)
' 05 ........ M04-Automatik ............. automatische Programmfortschaltung
' 06 ........ M05 ....................... Slave-Adressen in den EEprom schreiben
' 07 ........ M06 ....................... (frei)
' 08 ........ M07 ....................... (frei)
' 09 ........ M08 ....................... (frei)
' 10 ........ M09 ....................... (frei)
' 11 ........ M10 ....................... (frei)
'
'-LED
' blau ... rhythmisch blinkend ... : Anzeige des DCF-Empfanges
'                                    Ist dieser gestrt, erkennt man dies an
'                                    einer flackernden Daueranzeige.
' rot .... Sekundentakt .......... : Uhrzeit manuell
' grn ... Sekundentakt .......... : Uhrzeit DCF
'
'
'-------------------------------------------------------------------------------
A_020_historie:
'
'Versionsbersicht
'-----------------
'Label "A_15_lcdver": LCD-Anzeige der Versionsnummer aktualisiern
'
'
'
'
'
'
'
'
'V 2.0.13  05.08.2018 - Korrekturen . : - diverse Anpassungen (u. a. Kommentare)
'                                       - Verbesserung der Datenbertragung
'V 2.0.12  08.03.2018 - Korrekturen . : - Array komplett auslesen (Reihe)
'                     - Erweiterung . : - Programmerluterungen aktualisiert
'V 2.0.11  01.03.2018 - Test ........ : - Einflu des Bustaktes
'V 2.0.10  01.01.2018 - Erweiterung . : - Inhaltsverzeichnis aktualisiert
'V 2.0.09  31.12 2017 - Korrekturen . : - Programmwechsel optimiert
'                                       - bei Farbspiel Farb-Initialisierung
'                                         beim Start
'                                       - Anzeige und nderung bei LCD-Kontrast
'                                         und Strke der Hintergrundbeleuchtung
'V 2.0.08  30.12.2017 - Erweiterung . : - Slave-Adressen in das EEprom schreiben
'                                       - Programmwechsel im Handbetrieb +/-
'                                       - Optimierung der LCD-Anzeige
'V 2.0.07  27.12.2017 - Erweiterung . : - Aktivierung Zeitschaltuhr
'                                       - Variante variables Lauflicht ergnzt
'V 2.0.06  26.12.2017 - Erweiterung . : - Automatikbetrieb
'                                         Varianten eingearbeitet
'V 2.0.05  23.12 2017 - Erweiterung . : - Kommentare erweitert
'V 2.0.04  05.11.2017 - Korrekturen . : - LCD-Anzeigetexte und allmhliche n-
'                                         derung der Helligkeit beim Umschalten
'                                       - DCF in separaten Gehuse
'V 2.0.03  31.10.2017 - Erweiterung . : - DCF (kein Empang mglich)
'                                       - Ansteuerung der LED (Blinken rt/gn)
'V 2.0.02  30.10.2017 - Erweiterung . : - Drehschalterauswertung
'V 2.0.01  03.10.2017 - Erweiterung . : - Gerteaufbau
'          29.10.2017 -                 - Tastenauswertung
'V 1.1.01  26.03.2017 - nderung .... : - TWI-Anzahl Datenbyte: 1
'V 1.0.01  27.12.2015 - Information . : - Beginn der Softwareentwicklung
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'Programmspeicher des MCs (Programm-Flashspeicher): V 2.0.13 - 86%
'
'-------------------------------------------------------------------------------
'
'Informationen zur Software
'--------------------------
' Inhalt                                                                | Zeile
'-------------------------------------------------------------------------------
' 01. Allgemeine Informationen ........................................ |  222
' 02. Abkrzungsbersicht ............................................. |  237
' 03. Compilereinstellungen ........................................... |  243
' 04. Funktionsbeschreibung ........................................... |  266
' 05. Interrupt ....................................................... |  280
' 06. Einstellung der FUSE- und/oder LOCK-Bits ........................ |  286
' 07. Anschluschema fr den AVR ATmega16 ............................. |  329
' 08. I/O-Ports ....................................................... |  359
'      Abkrzungen fr die Signalbezeichnung .......................... |  361
' 09. EEprom-Belegung ................................................. |  459
' 10. Allgemein ....................................................... |  476
' 11. Schaltkreis ..................................................... |  481
'      Watchdog ....................................................... |  486
'      Schnittstelle .................................................. |  490
'      Compilereinstellung ............................................ |  495
' 12. Fuse- und Lockbits .............................................. |  503
' 13. Deklaration ..................................................... |  523
' 14. Konfiguration ................................................... |  655
'      I/O-Port ....................................................... |  657
'      Timer .......................................................... |  689
'      LCD-Modul ...................................................... |  721
'      DCF77 .......................................................... |  731
'      Interrupt ...................................................... |  744
' 15. Alias ........................................................... |  755
'      Eingang (digital) .............................................. |  759
'      Ausgang (digital) .............................................. |  776
' 16. Initialisierung ................................................. |  790
'      Konstanten ..................................................... |  794
'      LCD - Kontrast und HGB einstellen .............................. |  814
'      Lebenszeichen .................................................. |  835
'      Begrungsbildschirm ........................................... |  846
' 20. Programm ........................................................ | 1004
'      Initialisierung ................................................ | 1009
'      Allgemein ...................................................... | 1017
'      Abfrage Eingaben ............................................... | 1022
'      LCD-Initialisierung ............................................ | 1100
'      DCF Statusabfrage .............................................. | 1154
'      DCF Anzeige .................................................... | 1164
'      Programm festlegen ............................................. | 1212
'      LCD lschen .................................................... | 1225
' 30. Interruptroutine ................................................ | 1241
'      Timer 0 ........................................................ | 1249
'      Tastenauswertung ............................................... | 1303
'      PWM-Steuerung des LCD (Kontrast) ............................... | 1393
'      PWM-Steuerung des LCD (HGB Helligkeit) ......................... | 1400
'      Timer 2 ........................................................ | 1424
' 40. Programm ........................................................ | 1473
'      LCD-Initialisierung ............................................ | 1476
'      Tastenbettigung auswerten ..................................... | 1490
'      Drehgeber (DRS) auswerten ...................................... | 1605
'      Slave-Adressen ermitteln ....................................... | 1669
'      DCF77-Auswertung ............................................... | 1708
' 50. Unterprogramm-Call .............................................. | 1731
'      LCD-Ausgabe .................................................... | 1736
'      Daten an den Slave senden ...................................... | 1765
'      Pause mit Abfrage einer Tastenbettigung ....................... | 1781
' 60. Initialisierung der Slaves ...................................... | 1806
' 70. Funktion ........................................................ | 2119
' 80. Datenbereich .................................................... | 2123
' 90. SDcard .......................................................... | 2163
'      Code zur Anbindung einer Speicherkarte ......................... | 2166
'-------------------------------------------------------------------------------

'01. Allgemeine Informationen
'----------------------------
'Taktfrequenz
' Als Taktfrequenz wir eine Quarzfrequenz von 4,0 MHz verwendet.
' Werden die Bedingungen (die der TWI-Bus erfordert) eingehalten, kann auch
' mit einer geringeren Frequenz getaktet werden.
' Die Timer-Einstellungen sind dann anzupassen.
'
'Programmierung
' Die Programmierung des MCs kann extern oder ber die vorhandene Schnittstelle
' erfolgen.
' Zur Programmierung wurde das Modul "USBasp" verwendet, welches ber eine
' USB-Schnittstelle angeschlossen wird. Es kann direkt ber eine, in BASCOM
' integrierte Einstellung angesprochen werden.
'
'02. Abkrzungsbersicht
'-----------------------
' cq .... : Anruf an alle Busteilnehmer
' PWM ... : Pulsweitenmodulation (LCD-Kontrast)
' TWI ... : Two-wire Serial Interface (entspricht technisch dem I2C-Bus)

'03. Compilereinstellungen
'-------------------------
' Hinweise fr ATmega8/ATmega16 mit 1024 Byte SRAM
' Quelle:
' //halvar.at/elektronik/kleiner_bascom_avr_kurs/speicher_hwstack_swstack_frame/
'
' HW-Stack .. - Empfehlung: 40 Byte bei INT, sonst 10 Byte ausreichend
'  32 Byte fr INT + je GOSUB 2 Byte, wenn bei INT ein GOSUB erfolgt
'  (bei Verschachtelung ... + n x 2 Byte)
'
' SW-Stack .. - Empfehlung: 32 Byte (recht komplexe Aufrufe werden abgedeckt)
'   2 Byte fr jede bergebene Variable einer GOSUB-Routine + 2 Byte fr jede
'   LOCAL-Variable innerhalb einer GOSUB
'   (bei Verschachtelung ... n x 2 Byte)
'
' Frame-Space - Empfehlung: 60 Byte sollten alles abdecken
'                           (bei wenig Speicher verkleinern)
'  Beispiel:
'   2 x BYTE        =  2 Byte (erster und zweiter Parameter)
'   1 x WORD        =  2 Byte (dritte Parameter)
'   1 x STRING * 10 = 11 Byte (inkl. Abschlussbyte; vierter Parameter)
'   3 x BYTE        =  3 Byte (lokale Variablen)
'
'04. Funktionsbeschreibung
'-------------------------
' Allgemeines
' - - - - - -
' Die Anzahl der maximal ansteuerbaren RGB-LED hngt davon ab, inwieweit der
' beim I2C-Bus reservierte Adressbereich mit benutzt wird.
' Die Adressierung der Slaves erfolgt mit geraden Adressen und muss mit denen
' in den LED-Baugruppen bereinstimmen.
' Beim Anschluss des Microkontrollers gibt es keine Besonderheiten zu beachten.
' Die Takterzeugung erfolgt mit einem Quarz, die Programmierung ber die SPI-
' Schnittstelle. Am Port C.2 ist eine LED fr die Anzeige "Lebensbit" und
' an Port C.3 eine LED fr die Anzeige "Fehler bei der Datenbertragung" ange-
' schlossen.
'
'05. Interrupt
'-------------
' Timer 0:  8 Bit allgemeine Zeitintervall-Steuerung ... 1,0 ms
' Timer 1: 16 Bit DCF-Empfang .......................... 1,0  s
' Timer 2:  8 Bit berlauf-Interrupt ................... 4,1 ms (Drehschalter)
'
'06. Einstellung der FUSE- und/oder LOCK-Bits
'--------------------------------------------
' - FUSE-Einstellungen Brenner USBasp (Bascom)
'   Die Einstellungen sind im Abschnitt "Compilereinstellung" angegeben.
'
' - FUSE-bersicht
'   Die angegebenen HEX-Werte knnen direkt in die Brenner-Software eingetragen
'   werden, falls dies untersttzt wird (nicht bei USBasp unter Bascom).
'
' Bedeutung der Fuse/Lockbit-Einstellungen
' Name            MEGA16
'   Calibration 0   BD  (ohne Bedeutung)
'   Calibration 1   BC  (ohne Bedeutung)
'   Calibration 2   B7  (ohne Bedeutung)
'   Calibration 3   B7  (ohne Bedeutung)
' * Lockbits        FF
'    Lockbit 65       11: No restrictions for SPM or LPM accessing ...
'    Lockbit 43       11: No restrictions for SPM or LPM accessing ...
'    Lockbit 21       11: No memory lock features enabled for parallel ...
' * Fusebits        FA
'    Fusebit C         1: BODLEVEL 2.7V
'    Fusebit B         1: BODEN disabled
'    Fusebit KLA987   111010: Ext. Crystal/Resonator Low Freq.
' * Fusebits Hight  D1
'    Fusebit High M    1: Disable OCD
'    Fusebit High J    1: Disable JTAG
'    Fusebit High I    0: SPI enabled
'    Fusebit High H    1: CKOPT 1
'    Fusebit High G    0: Preserve EEPROM when chip erase
'    Fusebit High FE  00: 1024 word boot size, C00
'    Fusebit High D    1: Reset vector is $0000
'
' /// Achtung ///
' ===============
'Gltig fr MC mit folgenden Einstellmglichkeiten:
' - CKSEL
'   Wird CKSEL3= 0 gesetzt, arbeitet der MC nur noch mit einem externen RC-
'   Generator [f= 1/(3RC); C>= 22p].
'
' - RSTDISBL
'   Wird RSTDISBL= 1 gesetzt, ist mit einfachen Brennern kein Programmieren
'   mehr mglich, da dafr die Reset-Funktion unbedingt erforderlich ist.
'
'07. Anschluschema fr den AVR ATmega16
'---------------------------------------
A_070_avr_anschluss:
'                    -----------------|_|-----------------
' [1] T1       ---> | 1 [PB0]                     [PA0] 40| ---> LCD_RS  [x]
' [1] T2       ---> | 2 [PB1]                     [PA1] 39| ---> LCD_E   [x]
' [1] T3       ---> | 3 [PB2, INT2]               [PA2] 38| ---> LCD_DB4 [x]
' [0] TON      <--- | 4 [PB3, OC0]                [PA3] 37| ---> LCD_DB5 [x]
' [0] CS (SD)  <--- | 5 [PB4]                     [PA4] 36| ---> LCD_DB6 [x]
' [~] MOSI     ---> | 6 [PB5, MOSI]               [PA5] 35| ---> LCD_DB7 [x]
' [~] MISO     <--- | 7 [PB6, MISO]               [PA6] 34| ---> LCD_HGB [0]
' [~] SCK      ---> | 8 [PB7, SCK]                [PA7] 33| ---> LCD_KON [0]
' [~] /Reset   ---> | 9 [/Reset]                 [AREF] 32| (Anschluss 100 nF)
'          P5V ==== |10 [VCC]                    [AGND] 31| ==== GND
'          GND ==== |11 [GND]                    [AVCC] 30| ==== P5V
'           QZ ++++ |12 [XTAL2]                   [PC7] 29|
'           QZ ++++ |13 [XTAL1]                   [PC6] 28|
' [0] DIG_KA   ---> |14 [PD0, RXD]                [PC5] 27| ---> LED_3   [0]
' [0] DIG_KB   ---> |15 [PD1, TXD]                [PC4] 26| ---> LED_2   [0]
' [0] DIG_T    ---> |16 [PD2, INT0]               [PC3] 25| ---> LED_1   [0]
' [~] FU_S     <--- |17 [PD3, INT1]               [PC2] 24| ---> SEN     [x]
' [S] CS       <--- |18 [PD4]                [SDA, PC1] 23| ---> SDA     [#]
' [S] MOSI     <--- |19 [PD5]                [SCL, PC0] 22| ---> SCL     [#]
' [S] MISO     <--- |20 [PD6]                     [PD7] 21| ---> SCK     [S]
'                    -------------------------------------
'
' Pegel in []; 0/1 bei digitalen Signalen
'  ~: Impuls   0: aus   1: ein   #: TWI-Bussystem   x: unbestimmt
'  S: Soft-SPI

'08. I/O-Ports
'-------------
'Abkrzungen fr die Signalbezeichnung
'-------------------------------------
'Prfix
'~~~~~~
' /Reset  : Resetfunktion
' LED_    : Anschluss LED
' LCD_    : Anschluss LCD
' S_      : Sensor
' SDA     : Busanschluss (Daten)
' SCL     : Busanschluss (Takt)
' T<n>    : Taster (n: 1-3)
'
'Suffix
'~~~~~~
' _<n>    : diverse (n: 0-x); entsprechend der Funktion
' _A      : analog
' _DB<n>  : LCD-DB  (n: 4-7)
' _E      : LCD-E
' _FU     : Funkuhr
' _HGB    : LCD-Hintergrundbeleuchtung
' _KON    : LCD-Kontrasteinstellung
' _RS     : LCD-RS
'
'Allgemein
'~~~~~~~~~
' P5V_    : Stromversorgung P5 V
' GND     : Stromversorgung Masse
'
' MISO    : Programmieranschlu
' MOSI    : Programmieranschlu
' SCK     : Programmieranschlu
'
' QZ      : Anschlu Quarz
'
'           Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' -   : SV  P5V ........ | ............. | Stromversorgung .............. | 10
' -   : SV  GND ........ | ............. | Stromversorgung .............. | 11
' -   : SV  P5V ........ | ............. | Stromversorgung .............. | 30
' -   : SV  GND ........ | ............. | Stromversorgung .............. | 31
' -   : SV  P5V ........ | ............. | Referenzspannung ............. | 32
' -   : PRG /Reset ..... | ............. | Programmierung/Reset des MCs . |  9
' -   : QZ  Frequenz ... | ............. | Frequenznormal mit Quarz ..... | 12
' -   : QZ  Frequenz ... | ............. | Frequenznormal mit Quarz ..... | 13

' Port A    Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DA  LCD ........ | LCD_RS ...... | LCD-Signal RS ................ | 40
' - .1: DA  LCD ........ | LCD_E ....... | LCD-Signal E ................. | 39
' - .2: DA  LCD ........ | LCD_DB4 ..... | LCD-Datenbit 4 ............... | 38
' - .3: DA  LCD ........ | LCD_DB5 ..... | LCD-Datenbit 5 ............... | 37
' - .4: DA  LCD ........ | LCD_DB6 ..... | LCD-Datenbit 6 ............... | 36
' - .5: DA  LCD ........ | LCD_DB7 ..... | LCD-Datenbit 7 ............... | 35
' - .6: DA  LCD ........ | LCD_HGB ..... | LCD-Hintergrundbeleuchtung ... | 34
' - .7: DA  LCD ........ | LCD_KON ..... | LCD-Kontrasteinstellung ...... | 33
'
'Port B     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DE  T1 ......... | T1 .......... | Taste 1 ...................... |  1
' - .1: DE  T2 ......... | T2 .......... | Taste 2 LCD-Kontrast ......... |  2
' - .2: DE  T3 ......... | T3 .......... | Taste 3 LCD-Helligkeit_HGB ... |  3
' - .3: DA  Tonsignal .. | TON ......... | Signalisierung ............... |  4
' - .4: DA       SDcard  | CS .......... | Auswahl ........ SDcard ...... |  5
' - .5: PRG MOSI/SDcard  | MOSI ........ | Programmierung / SDcard ...... |  6
' - .6: PRG MISO/SDcard  | MISO ........ | Programmierung / SDcard ...... |  7
' - .7: PRG SCK /SDcard  | SCK ......... | Programmierung / SDcard ...... |  8
'
'Port C     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: TWI Takt  (Bus). | SCL ......... | TWI-Bussystem ................ | 22
' - .1: TWI Daten (Bus). | SDA ......... | TWI-Bussystem ................ | 23
' - .2: DA  Aktivierung  | SEN ......... | TWI-Treiber .................. | 24
' - .3: DA  Anzeige .... | LED_1 ....... | Anzeige (Busfehler) .......... | 25
' - .4: DA  Anzeige .... | LED_2 ....... | Anzeige (Status Funkuhr) ..... | 26
' - .5: DA  Anzeige .... | LED_3 ....... | .............................. | 27
' - .6: ..  ............ | ............. | .............................. | 28
' - .7: ..  ............ | ............. | .............................. | 29
'
'Port D     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DE  Drehgeber .. | DIG_KA ...... | Impulseingang Kanal A ........ | 14
' - .1: DE  Drehgeber .. | DIG_KB ...... | Impulseingang Kanal B ........ | 15
' - .2: DE  Drehgeber .. | DIG_T ....... | Impulseingang Taste .......... | 16
' - .3: DE  Funkuhr .... | FU_S ........ | Sensor DCF77 ................. | 17
' - .4: SB  SDcard ..... | CS .......... | SoftSPI SDcard ............... | 18
' - .5: SB  SDcard ..... | MOSI ........ | SoftSPI SDcard ............... | 19
' - .6: SB  SDcard ..... | MISO ........ | SoftSPI SDcard ............... | 20
' - .7: SB  SDcard ..... | SCK ......... | SoftSPI SDcard ............... | 21
'
'Erluterung
' DA : digitaler Ausgang
' DE : digitaler Eingang
' PRG: Programmierung
' QZ : Quarzanschluss
' SB : Soft-SPI
' SV : Stromverssorgung
' TWI: TWI-Bussystem

'09. EEprom-Belegung
'-------------------
A_090_eeprom:
'
'Adresse   | Inhalt
' Dez  Hex |
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' 001  01  | Anzahl der ermittelten Slave-Adressen (maximal 40)
'
' 002  02  | 1. Adresse im Bereich der Slave-Adressierung
' ...  ..  | ...
' 040  28  | Bereichsende (von der Gre des Arrays abhngig)
'
' 200  C8  | Wert fr den LCD-Kontrast
' 201  C9  | Wert fr die LCD-Hintergrundbeleuchtung
'
'-------------------------------------------------------------------------------
'10. === A L L G E M E I N =====================================================
'-------------------------------------------------------------------------------
'
A_110_schaltkreis:
'
'11. Schaltkreis
'---------------
   $regfile = "m16adef.dat"                       'Prozessor ....... : ATmega16A
   $crystal = 4000000                             'Quarzfrequenz ... : 4,0 MHz

'Watchdog
'--------
   Config Watchdog = 2048                         'Reset nach Timeout 2048 ms

'Schnittstelle
'-------------
' ggf. fr eine Ausgabe im Simulator-Feld UART
' $baud = 9600                                    'Baudrate ........ : 9600
                                                  '9600/19200/38400/57600/115200
'Compilereinstellung
'-------------------                              'bisher:
   $hwstack = 96                                  '--                                               'Hardware Stack
   $swstack = 64                                  '--                                               'Software Stack
   $framesize = 96                                '--                                                  'Frame Space

A_120_fuse:
'
'12. Fuse- und Lockbits
'----------------------
'USBasp: Werte generiert mit "Write PRG"
' $PROG &HFF, &HFA, &HD1, &H00   Byte 3: D9>D1: EEprom bei PRG nicht lschen
'
' FF: 11
' FA: 1 1 111010
' D9: 1 1 0 1 0 001
' 00: 0
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Archiv
' $PROG &HFF, &HFA, &HD1, &H00   Byte 3: D9>D1: EEprom bei PRG nicht lschen
' $PROG &HFF, &HFA, &HD9, &H00   Byte 2: FF>FA: Quarz-Frequenz Low
' $PROG &HFF, &HFF, &HD9, &HFF   Byte 3: D1>D9: EEprom bei PRG lschen
' $PROG &HFF, &HFF, &HD1, &H00
'
'-------------------------------------------------------------------------------
'
A_130_deklaration:
'
'13. Deklaration
'---------------
'Bibliothek (Betriebsart: TWI)
' Die Bibliothek ist Bestandteil von BASCOM.
  $lib "i2c.lbx"

'Subrotinen (Call)
'-LCD ansteuern
   Declare Sub Lcd_anz(byval Lcd_z1 As String * 16 , _
                       Byval Lcd_z2 As String * 16 , _
                       Byval Lcd_time As Byte)

'-Daten an einen Slave senden
   Declare Sub Twi_send(byval Aby As Byte , Byval Dby As Byte )

'-Wartezeit mit Unterbrechung bei einer Tastenbettigung
   Declare Sub Wait_tas(byval W_t As Byte , Byval W_b As Byte)

'Feld
   Dim Slv_wert(40)as Byte                        'Array(): Adressen der Slaves

'Flag
   Dim F_aut As Boolean                           'Automatikbetrieb (AB)
   Dim F_apw As Boolean                           'Programmwechsel im AB
   Dim F_cls As Boolean                           'LCD lschen
   Dim F_dcf As Boolean                           'Anzeige DCF
   Dim F_drs As Boolean                           'Anzeige Drehgeber
   Dim F_err As Boolean                           'DCF Fehler
   Dim F_hgb As Boolean                           'HGB (Ein/Aus der Beleuchtung)
   Dim F_hgb_dcf As Boolean                       'HGB nur 1x bei DCF
   Dim F_led As Boolean                           'LED Markierung
   Dim F_lcd As Boolean                           'LCD
   Dim F_t0c As Boolean                           'Tastendruck
   Dim F_tas As Boolean                           'Tastendruck
   Dim F_tw1 As Boolean                           'Tastenwert 1
   Dim F_tw2 As Boolean                           'Tastenwert 2
   Dim F_tw3 As Boolean                           'Tastenwert 3
'Flag (Fehler)
   Dim E_adr As Boolean                           'TWI-ADR nicht vorhanden

'Konstante
   Dim K_phgb As Byte                             'Wert PWM: LCD HGB
   Dim K_pkon As Byte                             'Wert PWM: LCD Kontrast
   Dim K_slvmax As Byte                           'max. Anzahl an Slaves
   Dim K_tdrs As Integer                          'Zeit DRS: Anzeigedauer
   Dim K_thgb As Integer                          'Zeit HGB: Leuchtdauer
   Dim K_epr As Byte                              'Wert 255

'Merker
   Dim M_adr As Byte                              'Adresse (TWI)
   Dim M_drs As Byte                              'Status
   Dim M_far As Byte                              'LED-Farbe
   Dim M_hgb As Byte                              'HGB der LCD
   Dim M_seq As Byte                              'Sendesequenz

'String
   Dim Lcd_alt1 As String * 16                    'LCD: alte Zeile 1
   Dim Lcd_alt2 As String * 16                    'LCD: alte Zeile 2

'Timer
   Dim T0_tve As Byte                             'Timer 0: Voreinstellwert
   Dim T2_tve As Byte                             'Timer 2: Voreinstellwert
   Dim T_lcd As Integer                           'LCD: Anzeige-Zeitdauer

'Variable
   Dim Led_f As Byte                              'LED-Farbe
   Dim Prg As Byte                                'LED-Programm
   Dim Prg_a As Byte                              'LED-Programm (bisher)
   Dim Pwm_kon As Byte                            'LCD: Kontrastwert (PWM)
   Dim Pwm_hgb As Byte                            'LCD: HGB-Wert (PWM)
   Dim T_twi As Byte                              'Wartezeit Datenbertragung
   Dim T_dcf As Long                              'DCF als Zahl
   Dim Tc As Byte                                 'Tastencode
   Dim Tc_alt As Byte                             'Tastencode (alt)
   Dim Tw As Byte                                 'Tastenwert

'Zhler
   Dim Z_apw As Byte                              'Zhler: Programmwechsel (AB)
   Dim Z_prg As Byte                              'Zhler: Programm manuell
   Dim Z_prg_a As Byte                            'Zhler: P m (alt)
   Dim Z_sec As Byte                              'Zeitzhler (SECTIC der DCF)
   Dim Z_t0a As Byte                              'Zeitzhler (T0: Zeitbasis A)
                                                  ' - LCD-HGB (Helligkeit)
   Dim Z_t0b As Byte                              'Zeitzhler (T0: Zeitbasis B)
                                                  ' - LCD-Kontrast
   Dim Z_t0c As Byte                              'Zeitzhler (T0: Zeitbasis C)
                                                  ' - Tasten-Klick
   Dim Z_t0d As Byte                              'Zeitzhler (T0: Zeitbasis D)
                                                  ' - Sekundentakt
   Dim Zt_drs As Integer                          'Zeitzhler Anzeige DRS ein
   Dim Zt_hgb As Integer                          'Zeitzhler HGB ein

   Dim Hz01 As Byte                               'Hilfszhler 01
                                                  ' - Slave-ADR       (Up_adr)
                                                  ' - Datenbertrag.  (Slv_prg)
                                                  ' - ...
   Dim Hz02 As Byte                               'Hilfszhler 02
                                                  ' - EEprom-ADR      (Up_adr)
                                                  ' - Twibyte         (Slv_prg)
                                                  ' - ...
   Dim Hz03 As Byte                               'Hilfszhler 03
                                                  ' - Farbermittlung
                                                  ' - ...
   Dim Hz04 As Byte                               'Hilfszhler 04
                                                  ' - ...

   Dim Hz_int As Integer                          'Hilfszhler Integer
   Dim Hz_wait As Byte                            'Hilfszhler Sub Wait_tas

   Dim Hs01 As String * 16                        'Hilfsstring
   Dim Hsf As String * 2                          'Hilfsstring Format "00"

   Dim Rz As Byte                                 'Zhler: Anzahl der Bytes
   Dim Ra As Byte                                 'Adresse: Slave
   Dim Rb As Byte                                 'Datenbyte
   Dim Rp As Byte                                 'Pause [ms]

'Drehschalter (Encoder)
   Dim Z_drs As Byte                              'Impulse Drehschalter
   Dim Z_drs_alt As Byte
   Dim Z_drw As Single

   Dim Drs_neu As Byte
   Dim Drs_alt As Byte
   Dim Drs_vr As Byte                             '1: DECR, 2: INCR, 0: ./.

'Konstante fr bedingte Compilation
'Drehimpulsgeber
   Const Doppelschritt = 1                        '0: alle zwei, 1: jedes Raster
'
'-------------------------------------------------------------------------------
'
'14. Konfiguration
'-----------------
'I/O-Port
'--------
A_141_port:
'
'=Port A
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddra = &B1111_1111                             'PA0-7 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Porta = &B0011_1111                            'PA0-5 aktiv; 6,7 passiv
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'=Port B
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrb = &B0000_0000                             'PB0-7 Ein
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portb = &B1111_0111                            'PB0-2,4-7 aktiv; 3 passiv
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'=Port C
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrc = &B1111_1100                             'PC0-7 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portc = &B000_0100                             'PC0-3 aktiv; 4-7 passiv
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'=Port D
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrd = &B1111_0000                             'PD0-3 Ein; 4-7 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portd = &B0000_1111                            'PD0-3 aktiv; 4-7 passiv
'
'-------------------------------------------------------------------------------
'
A_142_config:
'
'Timer
'-----
'/// Achtung ///
'Fr den Betrieb mit einer Funkuhr (DCF) ist Timer 1 erforderlich.
'Eine Konfiguration ist nicht erforderlich.

'Normal-Konfiguration
'Die Interrupts von Timer 0 werden beim jeweiligen Timerberlauf ausgelst.
'PSC: 1-8-64-256-1024
'PSC: Prescale; TZW: Timerzhlwert; TVE: Timervoreinstellwert

'T0:  0,001 s (1000 Hz)
'                                     PSC  TZW    T     f        TVE
'   T0 = 1/ Quarz* Teiler (1/ 16 MHz* 256*  63 =  1 ms;  992 Hz) 256-  63 = 193
'   T0 = 1/ Quarz* Teiler (1/  8 MHz*  64* 125 =  1 ms; 1000 Hz) 256- 125 = 131
' * T0 = 1/ Quarz* Teiler (1/  4 MHz*  64*  62 =  1 ms;  992 Hz) 256-  62 = 194
'   T0 = 1/ Quarz* Teiler (1/  2 MHz*   8*   6 =  1 ms; 1000 Hz) 256- 250 =   6

' * T2 = 1/ Quarz* Teiler (1/  4 MHz* 256*  62 =  1 ms;  992 Hz) 256- 191 = 191

'Systemzeit
'Timer 0 (8 Bit) voreinstellen auf Wert: TVE
   Config Timer0 = Timer , Prescale = 64          'Konfiguration
      T0_tve = 194                                'TVE: Voreinstellwert
      Timer0 = T0_tve                             'Timer 0: Voreinstellung

'Abfrage Drehschalter (4,1 ms)
'Timer2 voreinstellen auf Wert:   256-TVE (innerhalb der Interrupt-Routine)
   Config Timer2 = Timer , Prescale = 256         'Konfiguration
      T2_tve = 191                                'TVE: Voreinstellwert
      Timer2 = T2_tve                             'Timer 2: Voreinstellung

'LCD-Modul
'---------
'LCD Konfiguration (Modus 1)
'-LCD-Typ ..... : 2 Zeilen, 16 Zeichen
'-Datenport ... : Port C zur Datenbertragung erfordert fr das JTAG(EN)-Flag:
'                 Einstellung "disable" (FUSE-Bit)
   Config Lcd = 16 * 2
   Config Lcdpin = Pin , Db4 = Porta.2 , Db5 = Porta.3 , Db6 = Porta.4 , _
                         Db7 = Porta.5 , E = Porta.1 , Rs = Porta.0

'DCF77
'-----
'Funkuhr konfigurieren
'- jeder digitale Eingang ist fr den Signaleingang mglich
'- Timer 1 erforderlich
'  Er steht daher fr individuelle Aufgaben nicht zur Verfgung.
'- das DCF-Unterprogramm muss den Namen "Sectic" tragen
   Config Dcf77 = Pind.3 , Timer = 1 , Check = 0 , Inverted = 1 , Gosub = Sectic
'Datumsformat festlegen
   Config Date = Dmy , Separator = Dot
'
'-------------------------------------------------------------------------------
'
'Interrupt
'---------
   Enable Timer0                                  'Timer 0: Interrupt
      On Timer0 Int_tim0                          'Timer 0: berlauf (UP)
   Enable Timer2                                  'Timer 2: Interrupt
      On Timer2 Int_tim2                          'Timer 2: berlauf (UP)

   Enable Interrupts                              'Interrupts: global zulassen
'
'-------------------------------------------------------------------------------
'
'15. Alias
'---------
A_150_alias:
'
'Eingang (digital)
'-----------------
'Port B
   Taste Alias Pinb                               'Tastenabfrage
   Taste_1 Alias Pinb.0                           'Taste 1
   Taste_2 Alias Pinb.1                           'Taste 2
   Taste_3 Alias Pinb.2                           'Taste 3
'Port D
' Drehgeber
'  Wenn die Zhlrichtung des Drehgebers vertauscht werden soll, sind die Pins
'  entgegengesetzt zuzuordnen.
   Drs_a Alias Pind.1                             'Drehschalter Kanal A
   Drs_b Alias Pind.0                             'Drehschalter Kanal B
   Kt Alias Pind.2                                'Drehschalter Taste
' Funkuhr
   Fu_s Alias Pind.3                              'DCF-Signal

'Ausgang (digital)
'-----------------
'Port A
   Lcd_hgb Alias Porta.6                          'LCD: Hintergrundbeleuchtung
   Lcd_kon Alias Porta.7                          'LCD: Kontrast
'Port B
   Ton Alias Portb.3                              'Signalton
'Port C
   Led_twi Alias Portc.3                          'LED_rt: Fehler TWI (Bus)
   Led_leb Alias Portc.4                          'LED_gn: Lebensbit
   Led_mfu Alias Portc.5                          'LED_bl: Fehler Modul-Funkuhr
'
'-------------------------------------------------------------------------------
'
'16. Initialisierung
'-------------------
A_161_konstante:
'
'Konstanten
'----------
'LCD: maximaler Wert fr die HGB-Helligkeit
   K_phgb = 20
'LCD: maximaler Wert fr den Kontrast
   K_pkon = 20
'Leuchtdauer der HGB (T[s]x 1000)
   K_thgb = 30000                                 'Zeit: 30 s
'Anzeigedauer des Drehschalters (T[s]x 1000)
   K_tdrs = 10000
'Wert fr EEprom im Fehlerfall
   K_epr = 255

'Wartezeit nach einer Datenbertragung zur Fehlerreduzierung
' Sie wird im Programm automatisch korrigiert
   T_twi = 5                                      'Einheit [s]

'LED (0 V: Ein, => alle mit "1" ausschalten)
   Set Led_leb : Set Led_twi : Set Led_mfu

'LCD - Kontrast und HGB einstellen
'---------------------------------
'Die Werte fr die PWM-Steuerung beider Funktionen sind im EEprom abgelegt.
'ADR_KON: 200 [&HC8]; ADR_HBG: 201 [&HC9].

' Kontrast
   Readeeprom Pwm_kon , &HC8
      If Pwm_kon > 20 Then                        'Erststart: Wert 20 (&H14)
         Pwm_kon = 8                              'per Test ermittelt
         Writeeeprom Pwm_kon , &HC8               'EEprom: Wert [var, adr]
      End If

' Helligkeit (Hintergrundbeleuchtung)
   Readeeprom Pwm_hgb , &HC9
      If Pwm_hgb > 20 Then                        'Erststart: Wert 20 (&H14)
         Pwm_hgb = 20
         Writeeeprom Pwm_hgb , &HC9               'EEprom: Wert [var, adr]
      End If
'
'-------------------------------------------------------------------------------
'
'Lebenszeichen
'-------------
'Blinken beim Programmstart
'V 2.0.05 - besseres Verhalten bei deaktivierten Interrupts
Disable Interrupts
   For Hz01 = 1 To 6
      Led_leb = Not Led_leb                       'LED_gn (3x; Ein mit "0")
      Waitms 200
   Next Hz01
Enable Interrupts

'Begrungsbildschirm
'--------------------
'LCD-Initialisierung
A_162_lcdver:
'
   Gosub Up_lcd
   Call Lcd_anz( "Amatronik - LAD " , "Version: 2.0.13 " , 0)
'#HW_01
'Wenn nach einer LCD-Ausgabe eine weitere folgt, muss eine Wartezeit eingefgt
'werden.
'Die Angabe im Call-Aufruf bezieht sich auf ein automatisches Ausschalten der
'HGB [s].
   Wait 2

'Wartezeit vor dem Starten aller Buskomponenten
'FT1: Anzeige der Adressen der ermittelten Slaves
'FT2: zustzliche Ansteuerung der Slaves (RGB-LED-Modul; Farbe grn)
   Call Lcd_anz( "Buskomponenten ?" , "FT1-ADR  FT2-LED" , 0)
   For Hz01 = 1 To 30
      Waitms 100
      If F_tw1 = 1 Or F_tw2 = 1 Then              'bei Tastendruck Reduzierung
         Exit For                                 'der Zeit zur Auswahl
      End If
   Next Hz01

'Festlegung der Kanle fr die TWI-Betriebsart
   Config Scl = Portc.0                           'Kanal fr SCL (Port C.0)
   Config Sda = Portc.1                           'Kanal fr SDA (Port C.1)

A_163_bustakt:
'
'Der Befehl "Config Twi" setzt in Abhngigkeit der Taktfrequenz die Register
'TWSR und TWBR. Die Taktfrequenz der Slaves sollte mindestens das 16-fache der
'Busfrefrequenz betragen.
'Bei Adressierungsfehlern kann die Busfrequenz in bestimmten Grenzen reduziert
'werden. Noch geringere Busfrequenzen sind mglich, wenn die Taktfrequenz des
'MCs verringert wird.
'- bei 2 MHz bis 110000 erzielbar
'- bei 4 MHz bis 220000 erzielbar

   Config Twi = 50000                             '   Busfrequenz        [kHz]
                                                  '   - - - - - - - - - - - -
                                                  '   400000 ...... : 400 kHz
                                                  '   200000 ...... : 200 kHz
                                                  '   100000 ...... : 100 kHz
                                                  '    80000 ...... :  80 kHz
                                                  ' *  50000 ...... :  50 kHz
                                                  '    40000 ...... :  40 kHz
                                                  '    20000 ...... :  20 kHz
                                                  '    10000 ...... :  10 kHz
                                                  '     8000 ...... :   8 kHz

'01.03.2018 - 50000: bis 24 Elemente mit Kopplung ber USB-Verbindung
'01.08.2018 - 50000: bis 33 Elemente mit Kopplung ber LiYCY 4x0,25

   I2cinit                                        'I2C-Pins initialisieren
                                                  'auch nach PORT/DDR wichtig
   Waitms 200
'
'-------------------------------------------------------------------------------
'
'Slaves - Auslesen der Adressen der am Bus angeschlossenen Module
'----------------------------------------------------------------
'Die bei I2C-Bus reservierten Adressen werden nicht geprft.
'Adresse | R/W | Beschreibung                            | &Hxx   | &DEZ
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'0000000 |  0  | General Call Adresse                    |    00 |         0
'0000000 |  1  | Startbyte                               |    01 |         1
'0000001 |  X  | CBUS Adresse                            | 02-03 |   2 -   3
'0000010 |  X  | Reserviert fr ein anderes Busformat    | 04-05 |   4 -   5
'0000011 |  X  | Fr zuknftige Erweiterungen reserviert | 06-07 |   6 -   7
'00001XX |  X  | Fr zuknftige Erweiterungen reserviert | 08-0F |   8 -  15
'11110XX |  X  | 10-Bit Adressierung                     | 70-F7 | 112 - 247
'11111XX |  X  | Fr Zuknftige Erweiterungen Reserviert | F8-FF | 248 - 255

'Erstinitialisierung
'/// Hinweis ///
'Wenn "Programm startet" in der Anzeige nicht erscheint, kann es daran liegen,
'dass in der Schaltung die Busabschluss-Widerstnde nicht aktiv sind.
'Wird der Bustreiber verwendet, sind vor und hinter diesem die Widerstnde ber
'die entsprechenden Jumper zu aktivieren.

Disable Timer2
    I2cstart : Wait 2 : I2cstop                   'zur Synchronisierung
    Waitms 100

    Call Twi_send(&H00 , 254)                     'cq, 254: Modus aus
    Call Twi_send(&H00 , 252)                     'cq, 252: LED aus

'Ermittlung aller Slaves
'Bei gewhlter Option (FT1) werden die Adressen angezeigt.
'Adress-Bereich: &H22 -> &H6E = 34 -> 110
 For Hz01 = &H22 To &H6E Step 2                   'Adressbereich durchlaufen
      I2cstart                                    'Senden: Start
         Waitms 100
      I2cwbyte Hz01                               'Senden: Adresse
         Waitms 100
      If Err = 0 Then                             'Adressierung mglich
'Adresse in das Array schreiben
         Incr Hz02                                'Arraywert 1
         Slv_wert(hz02) = Hz01                    'Array beschreiben
         Incr K_slvmax                            'Zhler: Array-Elemente
      Else
         If E_adr = 0 Then                        'noch kein Fehler aufgetreten
            Set E_adr                             'Set Flag: Fehler TWI-ADR
            M_adr = Hz01                          'Adresse merken
         End If
      End If
      I2cstop                                     'Bus freigeben
      Waitms 100

'Mit FT1 wird die Anzeige der Adressen und deren Status aktiviert.
'Mit FT2 werden zustzlich die LED eingeschaltet (grn) und die Anzeigezeit
'verlngert.
      If F_tw1 = 1 Or F_tw2 = 1 Then              'Flag Taste 1/2
         If Err = 0 Then
            Hs01 = "ADR..: ok"
         Else
            Hs01 = "ADR..: nc"
         End If

         Call Lcd_anz( "Slave: " + Str(hz01) , Hs01 , 0) : Waitms 300

'LED einschalten (grn)
         If F_tw2 = 1 Then
            Call Twi_send(hz01 , 2)               'ADR,   2: Farbe grn
            Call Twi_send(hz01 , 251)             'ADR, 251: LED ein
         End If

         Call Lcd_anz( "Slave: " + Str(hz01) , "ADR..: --" , 0) : Waitms 30
      Else
         Locate 1 , 16                            'LCD-Position in Reihe 1
         Toggle F_lcd
         If F_lcd = 1 Then
            Lcd ">"                               'Anzeige Durchlauf
         Else
            Lcd "<"
         End If
      End If
   Next Hz01

'Tasten-Flags nicht mehr erforderlich
   Reset F_tw1 : Reset F_tw2 : Reset F_tw3        'Reset Flag Taste 1-3

'Auswertung der Adress-Ermittlung
   Select Case M_adr
      Case &H22                                   'nur ersten Slave erkannt
         Hs01 = "TWI-Bus defekt?"                 'nach dem ersten Slave
      Case Is > &H22                              'erste nicht belegte Adresse
         M_adr = M_adr - 2
         Hs01 = "-> ADR " + Str(m_adr) + " - " + Str(hz02) + " E"
   End Select

   Call Lcd_anz( "Programm startet" , Hs01 , 0)
   Wait 2                                         'siehe Hinweis: #HW_01
Enable Timer2
'
'-------------------------------------------------------------------------------
'20. === P R O G R A M M =======================================================
'-------------------------------------------------------------------------------
'
A_210_init:
'
'Initialisierung
'---------------
   Z_drs = 1                                      'Start: Zeiteinstellung

A_220_programm:
'
Do

'Allgemein
'---------
'Einschalten des Automatikbetriebes in der Zeit von 15:30-21:30 oder ber den
'Drehschalter in Stellung "M04-Automatik" (FT1: Ein, FT2: Aus)

'Abfrage Eingaben
'----------------
   Gosub Up_tas                                   'Tasten
   Gosub Up_drs                                   'Drehgeber

A_230_time:
'
'/// Achtung ///
'Beim direkten Zugriff auf <Time$> funktioniert das Programm nicht korrekt.
   Hs01 = Time$                                   'Zeit
   Delchars Hs01 , ":"                            'Doppelpunkt entfernen
   T_dcf = Val(hs01)                              'Zahlenwert der Zeit

'Der Automatikbetrieb wird ber zwei Zeitpunkte geschaltet.
'Soll der manuelle Betrieb in einem Zeitfenster gesperrt werden, ist der aus-
'kommentierte Code zu verwenden.
'Zeitpunkte
   If T_dcf = 153000 Then                         'Ein: 15:30
      Z_drs = 4                                   'LCD einstellen
      Set F_aut                                   'Set Flag Automatik
   End If

   If T_dcf = 213000 Then                         'Aus: 21:30
      Reset F_aut                                 'Reset Flag Automatik
   End If

'Zeitfenster (siehe Information "Zeitpunkte")
'Im Modus "Zeitfenster" ist eine manuelle Bedienung nicht mglich, da stets in
'den Modus "Automatik" umgeschaltet wird
'   If T_dcf > 153000 And T_dcf < 213000 Then      'Zeitfenster: 15:30 - 21:30
'      Set F_aut                                   'Set Flag Automatik
'   Else
'      Reset F_aut                                 'Reset Flag Automatik
'   End If

'Im Modus "Automatik" erfolgt der Programmwechsel alle 30 Sekunden.
'Der Zeitzhler befindet sich in der DCF-Auswertung (Sectic).
   If F_aut = 1 And F_apw = 1 Then                'Automatik und Programmwechsel
'Ausschaltmglichkeit ber FT2
      Reset F_apw                                 'Reset Flag
      Z_prg = 255                                 'ggf. fr LCD-Anzeige
      Incr Z_apw                                  'nchste Programm

'+++ Test +++
'-Beschrnkung auf einen Bereich
'>
'      If Z_apw < 57 Then Z_apw = 57
'      If Z_apw > 63 Then Z_apw = 57
'<
'
'Bereich der zu durchlaufenden Programm-Abschnitte

      If Z_apw > 63 Then Z_apw = 1                'Begrenzung: <Z_apw>
'Die Anzahl der Farben betrgt 7, die alle je nach Version durchlaufen werden.
'                                 Z_apw       PRG
'Farbe Ein/Aus ..................  1 -  7 .... 1 -  7
'Farbwechsel, Blitz .............  8 - 14 ... 28 - 34
'Farbwechsel, Helligkeit ........ 15 - 21 ... 35 - 41
'Farbreihe vorwrts ............. 22 - 28 ... 42 - 48
'Farbspiel ...................... 29 - 35 ... 49 - 55
'Farbreihe rckwrts ............ 36 - 42 ... 56 - 62
'zwei Farben wechselnd .......... 43 - 49 ... 63 - 69
'drei Farben wechselnd .......... 50 - 56 ... 70 - 76
'Lauflicht ...................... 57 - 63 ... 77 - 83

'Fr den Programmwechsel wird eine neue Steuersequenz bertragen.
'Dazu wird <Z_prg> mit dem Programmcode initialisiert.
'Programme: siehe Label "A_610_slave"
'Korrekturwert
      If Z_apw < 8 Then
         Prg = Z_apw                              'ergibt LED-Farbe rt -> ws
      Else
         Prg = 20 + Z_apw
      End If
  End If

A_240_lcd:
'
'LCD-Initialisierung
'-------------------
   Hs01 = "(ohne Funktion)"

   Select Case Z_prg
      Case 0                                      'Case <Z_prg>
         Hs01 = " (Programmwahl)"
      Case 1 To 8                                 'auch fr Automatikbetrieb
         M_far = Z_prg - 1                        'Korrektur
         Hs01 = Lookupstr(m_far , Dat_far)
         Hs01 = "Farbe: " + Hs01
      Case 9                                      'Case <Z_prg>
         Prg = 0                                  'ohne Funktion
      Case 10                                     'Case <Z_prg>
         Hs01 = "Flackern (gelb)"                 'Flackerlicht "gelb"
      Case 11
         Hs01 = "Blitz (gelb)"                    'Grundhelligkeit mit Blitz
      Case 12
         Hs01 = "HK-Wechsel"                      'Wechsel der LED-Helligkeit
      Case 13
         Hs01 = "Reihe ein (rot)"                 'nacheinander einschalten

      Case 14 To 27                               'Case <Z_prg>
         Prg = 0                                  'ohne Funktion
'Bereich Automatikbetrieb                         'alle Farben nacheinander (n)
      Case 28 To 34                               'Case <Z_prg>
         Hs01 = "Farbe n, Blitzen"
      Case 35 To 41
         Hs01 = "Farbe n, HW"                     'HW: Helligkeitswechsel
      Case 42 To 48
         Hs01 = "Farbe n, Reihe v"                'v: vorwrts
      Case 49 To 55
         Hs01 = "Farbspiel"                       'bunt
      Case 56 To 62
         Hs01 = "Farbe n, Reihe r"                'r: rckwrts
      Case 63 To 69
         Hs01 = "Farb-W, 2 LED"                   'Farbwechsel mit 2 Farben
      Case 70 To 76
         Hs01 = "Farb-W, 3 LED"                   'Farbwechsel mit 3 Farben
      Case 77 To 83
         Hs01 = "Lauflicht div."                  'Gruppen diverser Lnge

      Case 84 To 99                               'Case <Z_prg>
          Prg = 0                                 'ohne Funktion
      Case 100                                    'Case <Z_prg>
         Hs01 = "(alle LED  aus)"                 'alle ausschalten

      Case Else                                   'Case <Z_prg>
         Z_prg = 0                                'Anfangswert

   End Select

A_250_dcf:
'
'DCF Statusabfrage
'-----------------
'Bit 4 - H: DCF-Teil hat gestoppt
'Bit 7 - L: keine Synchronisation
   If Dcf_status.4 = 1 Or Dcf_status.7 = 0 Then
      Set F_err
   Else
      Reset F_err
   End If

'DCF Anzeige
'-----------
'Die Anzeige im LCD hngt vom Wert des Drehschalters ab.
   Select Case Z_drs
 'Programmauswahl fr die LED-Steuerung
      Case 0                                      'Case <Z_drs>
         Hsf = Str(z_prg)
         Hsf = Format(hsf , "00")
         Call Lcd_anz( "M00-PG: " + Hsf + " FT1/2" , Hs01 , 0)
 'Anzeige von Zeit und Datum
      Case 1                                      'Case <Z_drs>
         If F_err = 1 Then
               Hs01 = "   (FTx)"                  'bei DCF-Fehler 00:00:00 (FTx)
            Else
               Hs01 = "   (DCF)"
         End If
         Call Lcd_anz(time$ + Hs01 , Date$ , 0)
'LCD: Kontrast
      Case 2                                      'Case <Z_drs>
         Hs01 = Str(pwm_kon)
         Hs01 = Format(hs01 , "00")
         Hs01 = Hs01 + "  LCD-Kontrast"
         Call Lcd_anz( "M02-Wert     FT1" , Hs01 , 0)
'LCD: Helligkeit
      Case 3                                      'Case <Z_drs>
         Hs01 = Str(pwm_hgb)
         Hs01 = Format(hs01 , "00")
         Hs01 = Hs01 + "  LCD-HGB"
         Call Lcd_anz( "M03-Wert     FT1" , Hs01 , 0)
'Automatik Ein/Aus
      Case 4                                      'Case <Z_drs>
         If F_aut = 0 Then
            Hs01 = "FT1: Ein"
         Else
            Hs01 = "        FT2: Aus"
         End If
         Call Lcd_anz( "M04-Automatik" , Hs01 , 0)
'gltige Slave-Adressen in das EEprom schreiben
      Case 5                                      'Case <Z_drs>
         Hs01 = "M05-TWI " + Str(k_slvmax) + " E FT1"       'Anzahl der Slaves
         Call Lcd_anz(hs01 , "EEprom Slave-ADR" , 0)

      Case Else                                   'Case <Z_drs>
         Hsf = Str(z_drs)
         Hs01 = "M" + Format(hsf , "00") + "          FT1"
         Call Lcd_anz(hs01 , "-(frei)-" , 0)
   End Select

'Programm festlegen
'------------------
'Bedingungen fr UP-Aufruf:
'neues Programm oder andere Variante (Farbe im Automatikbetrieb)
   If Prg_a <> Prg Or Z_prg_a <> Z_prg Then
      Prg_a = Prg                                 'Durchlauf: 1x
      Z_prg_a = Z_prg
'Reset Slaves
      Call Twi_send(&H00 , 252)                   'cq, 252: LED aus
      Call Twi_send(&H00 , 254)                   'cq, 254: Modus aus
      Gosub Slv_prg                               'LED-Programm
   End If

'LCD lschen
'-----------
'Das LCD wird gelscht, wenn beim Aufruf eine Anzeigezeit > 0 eingetragen wurde.
   If F_cls = 1 Then
      Cls
      Reset F_cls
   End If

M_ende:
'Ende Programmschleife
Loop

'Ende Programm
End

'-------------------------------------------------------------------------------
'30. === I N T E R R U P T R O U T I N E =======================================
'-------------------------------------------------------------------------------
'
' /// Achtung ///
' Der Timer 1 ist fr die Funkuhr (DCF77) reserviert.
'
A_310_int_t0:
'
'Timer 0
'-------
Int_tim0:
'   T0 = 1/ Quarz* Teiler (1/ 16 MHz* 256* 193 =  1 ms;  992 Hz)
'   T0 = 1/ Quarz* Teiler (1/  8 MHz*  64* 125 =  1 ms; 1000 Hz)
' * T0 = 1/ Quarz* Teiler (1/  4 MHz*  64*  62 =  1 ms;  992 Hz)
'   T0 = 1/ Quarz* Teiler (1/  2 MHz*   8*   6 =  1 ms; 1000 Hz)

'Interrupt: Timer 0 (8 Bit), alle 1 ms
'Voreinstellung des Zhlers
   Timer0 = T0_tve                                'Voreinstellung
   Incr Z_t0a : Incr Z_t0b                        'Zeitbasis (A, B)

'Mit einem anderen Endwert kann die Anzahl der Schritte fr den Kontrast oder
'die Hintergrundhelligkeit des LCDs verndert werden.
'Zeitbasis: 20 ms
   If Z_t0a = 20 Then                             'Zhler A: LCD-Kontrast
      Z_t0a = 0
      Reset Watchdog                              'PRG-Laufzeitberwachung
   End If

'Zeitbasis: 20 ms
   If Z_t0b = 20 Then                             'Zhler B: LCD-HGB (Strke)
      Z_t0b = 0
   End If

'LED (Sekundenanzeige)
   If Z_drs <> 1 Then                             'Sekundentakt bei Modus <> 1
      If F_dcf = 1 Then
         If F_err = 0 Then
            Reset Led_leb                         'LED_gn ein
         Else
            Reset Led_twi                         'LED_rt ein
         End If
         Incr Z_t0d
      End If
      Set Led_mfu                                 'LED_blau aus
   Else
'Das Flackerlicht (LED_blau) zeigt das DCF-Empfangssignal am AVR an.
'Bei einem flackernden Dauerlicht ist der Empfang gestrt; die Strquellen
'sind zu beseitigen.
'Zu beachten ist je nach Empfangslage die Zeit, die das DCF-Modul bentigt, um
'sich mit dem DCF-Signal abzustimmen, bis das Zeitsignal empfangen werden kann.
'Sie kann bis zu 20 Minuten (DCF1; Fa. Pollin) betragen.
      Led_mfu = Fu_s
   End If

   If Z_t0d = 2 Then                              'Anzeigedauer 2 ms
      Z_t0d = 0
      Reset F_dcf                                 'Reset Flag (Set UP 'Setic')
      Set Led_leb                                 'LED_gn aus
      Set Led_twi                                 'LED_rt aus
   End If

'Tastenauswertung
'----------------
'Wenn bereits eine Taste gedrckt ist, erfolgt der Tastenklick beim Ein- und
'Ausschalten der Taste.
'nderung ermitteln
   Tc = Taste And &B0000_0111                     'Tastenabfrage (Bit 0-2)

   If Z_t0c = 0 And Tc <> Tc_alt Then             'Tastencode verndert
      Tc_alt = Tc                                 'Tastencode merken
      Set F_t0c                                   'Set Flag_Zeitzhler
   End If

'Zeitzhler Taste
   If F_t0c = 1 Then Incr Z_t0c                   'Zhlen Tastenzeit

'Entprellen (20 ms)
   If Z_t0c = 20 And Tc < 7 Then                  't= 20 ms, Tastencode gltig
      Set Ton                                     'Ton ein (Tastenklick)
   End If

'Tastenklick (2 ms)
   If Z_t0c = 22 Then                             't= 22 ms
      Reset Ton                                   'Ton aus,Dauer: 22- 20= 2 ms
      If Tc = Tc_alt Then                         'Tastencode stabil
         Set F_tas                                'Set Flag Taste
'Tastenwert <Tw> ermitteln
         Select Case Tc
            Case 6
               Tw = 1                             'Taste 1
               Set F_tw1                          'Flag Taste 1
            Case 5
               Tw = 2                             'Taste 2
               Set F_tw2                          'Flag Taste 2
            Case 3
               Tw = 3                             'Taste 3
               Set F_tw3                          'Flag Taste 3
            Case 4
               Tw = 12                            'Taste 1/2
            Case 2
               Tw = 13                            'Taste 1/3
            Case 1
               Tw = 23                            'Taste 2/3
            Case 0
               Tw = 123                           'Taste 1/2/3
           Case Else
               Reset F_tas
         End Select
      End If
   End If

   If Z_t0c = 30 Then
      Reset F_t0c                                 'Reset Flag Tastenzhler
      Z_t0c = 0                                   'Zeitzhler Taste= 0
   End If

'LCD-HGB und LCD-Anzeige 'Programm'
'- aktiviert mit Tastenbedienung
   If Tw > 0 Then                                 'HGB-LCD ein bei Tastendruck
      Set F_hgb
   End If

'- aktiviert mit UP 'LCD_anz' bzw. Tastenbedienung
   If F_hgb = 1 Then                              'Flag: Anforderung HGB
      Reset F_hgb                                 'Reset Flag
      Zt_hgb = 1                                  'Start Anzeigedauer
   End If

   If Zt_hgb > 0 Then Incr Zt_hgb                 'Anzeigedauer weiterzhlen

   If Zt_hgb >= K_thgb Then                       'max. Zeitdauer erreicht
      Zt_hgb = 0
   End If

   If F_drs = 1 Then
      Incr Zt_drs                                 'Zeitzhler Anzeige DRS ein
      If Zt_drs = K_tdrs Then
         Reset F_drs
         Zt_drs = 0
      End If
   End If

'LCD-Anzeige lschen
   Select Case T_lcd
      Case Is > 1
         Decr T_lcd                               'Anzeigezeit reduzieren
      Case 1
         Set F_cls                                'LCD lschen
         Decr T_lcd
   End Select

'PWM-Steuerung des LCD (Kontrast)
'--------------------------------
'Eine lngere Einschaltzeit bedeutet eine hhere Spannung und damit einen ge-
'ringeren Kontrast des LCDs.
   If Z_t0a = 0 Then Set Lcd_kon                  'LCD-Kontrast: Ein
   If Z_t0a >= Pwm_kon Then Reset Lcd_kon         'LCD-Kontrast: Aus

'PWM-Steuerung des LCD (HGB Helligkeit)
'--------------------------------------
'Ein lnger PWM-Einschaltwert bedeutet eine hellere Hintergrundbeleuchtung.
'Der Helligkeitswechsel beim Ein- bzw- Ausschalten erfolgt allmhlich.
   If Z_t0b = 0 Then                              'zum Zeitpunkt 0
      If Zt_hgb > 0 Then
'Helligkeit bis <Pwm_hgb> erhhen, wenn noch nicht erfolgt.
         If M_hgb < Pwm_hgb Then Incr M_hgb       'alle 20 ms
      Else
'Helligkeit bis <M_hgb> = 0 verringern.
         If M_hgb > 0 Then Decr M_hgb             'alle 20 ms
      End If
      If M_hgb > 0 Then Set Lcd_hgb               'LCD-Beleuchtung PWM: Ein
   End If

   If Z_t0b >= M_hgb Then
      Reset Lcd_hgb                               'LCD-Beleuchtung PWM: Aus
   End If

Int0_ret:
Return

A_320_int_t2:
'
'Timer 2
'-------
'Mit diesem Timer wird der Drehgeber ausgewertet. Der Timer wird zwischenzeit-
'lich im Hauptprogramm deaktiviert.
Int_tim2:
'Interrupt: Timer2 (8 Bit); Aufruf alle 4,1 ms
'Voreinstellung des Zhlers
   Timer2 = T2_tve                                'Voreinstellung
   If F_aut = 1 Then Goto Int2_ret                'Automatik: keine Auswertung

'Whrend der Datenausgabe wird der Timer gestoppt.
'Ausgabe <Enc_vr>: 2 rechts, 1 links (bei Flankenwechsel)
'Wird der Wert in der Hauptroutine verwendet, ist er dort zurckzusetzen.
'Schaltzustnde (Encoder: ALPS STEC11; Fa. Reichelt):
'vorwrts                           rckwrts
' A    B                             A     B
' 1    1  rastende Startposition     1     1
' 1    0  mittlere Position 1        0     1
' 0    0  mittlere Position 2        0     0
' 0    1  mittlere Position 3        1     0
' 1    1  nachste rastende Position  1     1

   Drs_neu.0 = Drs_a                              'Pinabfrage Kanal A
   Drs_neu.1 = Drs_b                              'Pinabfrage Kanal B

'Abfrage der Encoderzustnde bei einem Flankenwechsel
'Enc_vr = 1: vorwrts
'Enc_vr = 2: rckwrts
   If Drs_neu <> Drs_alt Then                     'Wechsel am Encoder
'von der Zwischenposition 3 in die Ruheposition
      If Drs_neu = &B00000011 And Drs_alt = &B00000010 Then Drs_vr = 2
      If Drs_neu = &B00000011 And Drs_alt = &B00000001 Then Drs_vr = 1
'Bedingte Compilation
'0: ein  Schritt  beim Flankenwechsel
'1: zwei Schritte beim Flankenwechsel (langsames Drehen)
     #if Doppelschritt = 1
        If Drs_neu = &B00000000 And Drs_alt = &B00000001 Then Drs_vr = 2
        If Drs_neu = &B00000000 And Drs_alt = &B00000010 Then Drs_vr = 1
     #endif
      Drs_alt = Drs_neu                           'Wert bernehmen
'Die Reaktion auf Tastendruck nach DRS-nderung erfolgt mit folgendem Eintrag
'korrekter.
      Tc_alt = 0
   End If

Int2_ret:
Return

'-------------------------------------------------------------------------------
'40. === U N T E R P R O G R A M M =============================================
'-------------------------------------------------------------------------------
'
'LCD-Initialisierung
'-------------------
'Init ist erforderlich, da es manchmal zu Fehlanzeigen kommt.
A_410_uplcd:
'
Up_lcd:
   Disable Interrupts
      Initlcd                                     'Initialisierung
      Cls
      Cursor Off                                  'Cursor ausschalten
   Enable Interrupts

Return

'Tastenbettigung auswerten
'--------------------------
A_420_uptas:
'
Up_tas:
'
'Taste    | FT1                     | FT2                   | FT3
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Modus 0 | Programmfortschaltung + | PFS -                 | Programm-Abbruch
' Modus 1 | Wert: Stunde            | Wert: Minute          | Sekunden = 0
' Modus 2 | LCD-Kontrast          + | LCK -                 | %
' Modus 3 | LCD-Helligkeit        + | LCH -                 | %
' Modus 4 | Automatkbetrieb ein     | Automatkbetrieb aus   | Programm-Abbruch
' Modus 5 | Slave-Adresse schreiben | %                     | %

'Abkrzung
'+/- - erhhen/verringern
'LCK - LCD-Kontrast
'LCH - LCD-Helligkeit
'PFS - Programmfortschaltung

   Select Case Z_drs
      Case 0                                      'Case <Z_drs>
'Taste 1: Programmfortschaltung manuell +
         If Tw = 1 Then
            Incr Z_prg
            If Z_prg > 100 Then Z_prg = 1
         End If
'Taste 2: Programmfortschaltung manuell -
         If Tw = 2 And Z_prg > 1 Then
            Decr Z_prg
         End If

         Select Case Z_prg
            Case 1 To 83
               Prg = Z_prg
            Case 100
               Prg = 254                          'Status: alle LED aus
         End Select

      Case 1                                      'Case <Z_drs>
'Wenn keine DCF-Zeit angezeigt wird, kann mit den Tasten FT1-3 die Zeit einge-
'stellt werden.
'FT1: Stunden; FT2: Minuten; FT3: Sekunden= 00
         If Mid(lcd_alt1 , 13 , 1) = "F" Then     'Anzeige: 00:00:00    (FTx)
            Select Case Tw
               Case 1                             'FT1: Stunden
M_t1:
                  If _hour < 23 Then
                     Incr _hour
                     Hs01 = Str(_hour)
                     Locate 1 , 1 : Lcd Format(hs01 , "00")
                     Set Ton : Waitms 20 : Reset Ton : Waitms 200
                     Set F_hgb
                     If Taste_1 = 0 Then Goto M_t1
                  Else
                     _hour = 0
                  End If
               Case 2                             'FT2: Minuten
M_t2:
                  If _min < 59 Then
                     Incr _min
                     Hs01 = Str(_min)
                     Locate 1 , 4 : Lcd Format(hs01 , "00")
                     Set Ton : Waitms 20 : Reset Ton : Waitms 200
                     Set F_hgb
                     If Taste_2 = 0 Then Goto M_t2
                  Else
                     _min = 0
                  End If
                Case 3                            'FT3: Sekunden= 00
                  _sec = 0
            End Select
         End If
      Case 2                                      'Case <Z_drs>
'Taste 1: Einstellung LCD-Kontrast
         If Tw = 1 Then
            If Pwm_kon < K_pkon Then
               Incr Pwm_kon
               Writeeeprom Pwm_kon , &HC8         'EEprom: Wert [var, adr]
            Else
               Pwm_kon = 0
            End If
         End If
      Case 3                                      'Case <Z_drs>
'Taste 1: Einstellung HGB-Helligkeit
         If Tw = 1 Then
            If Pwm_hgb < K_phgb Then
               Incr Pwm_hgb
               Writeeeprom Pwm_hgb , &HC9         'EEprom: Wert [var, adr]
            Else
               Pwm_hgb = 0
            End If
            M_hgb = 0                             'Initialisierung HGB
         End If
'Autoamtikbetrieb Ein/Aus
      Case 4                                      'Case <Z_drs>
         If Tw = 1 Then                           'Automatik ein
            Set F_aut
            Z_sec = 59                            'Automatik beginnt schneller
            Prg = 1                               'Start mit Programm 1
         End If
         If Tw = 2 Then
            Reset F_aut                           'Automatik aus
         End If
'EEprom mit den Slave-Adressen beschreiben
      Case 5                                      'Case <Z_drs>
         If Tw = 1 Then
            Gosub Up_adr                          'UP aufrufen
         End If
    End Select

   Tw = 0
Return

'Drehgeber (DRS) auswerten
'-------------------------
A_430_updrs:
'
Up_drs:
'Im Automatikbetrieb ist der DRS deaktiviert.
   If F_aut = 1 Then Goto Drs_end

'Auswertung der Abfrage; der Timer 2 ist dazu deaktiviert.
Disable Timer2                                    'kein Interrupt Timer 2
   Select Case Drs_vr                             'richtungsabhngiges Zhlen
      Case 2                                      'vorwrts
         If Z_drs < 10 Then
            Incr Z_drs
         End If
      Case 1                                      'rckwrts
         If Z_drs > 0 Then
            Decr Z_drs
         End If
         Z_prg = 0                                'Reset LED-Programm bei DRS
   End Select

'aktuelle Richtungsangabe lschen
   If Drs_vr > 0 Then
'Bereich fr Code, wenn sich die Richtung gendert hat
'>
'
'<
      Drs_vr = 0
'LCD-Hintergrundbeleuchtung
      Set F_hgb                                   'Set Flag_HGB
'Tonausgabe
'Dazu werden Flag und Zhler der Tastenabfrage verwendet.
      Set F_t0c
      Z_t0c = 20
      Set Ton
'Steuerung der Anzeige
'<z_drs>: Wert des Drehgebers (Modus)
      Set F_drs
   End If
Enable Timer2                                     'Interrupt Timer 2 zulassen

'DRS-Bettigung: alle LED lschen
'Bei Automatikbetrieb erfolgt der Rcksprung bereits beim Code-Anfang.
   If F_drs = 1 Then
      Call Twi_send(&H00 , 252)                   'cq, 252: LED aus
   End If

'Besttigungstaste des Drehgebers
   If Kt = 0 Then
      Set Ton : Waitms 50 : Reset Ton             'Piep bei Besttigung (50 ms)
M_kt:
      If Kt = 0 Then Goto M_kt
'Code fr die Auswertung der Bettigungstaste
'>
'
'<
      M_drs = Z_drs                               'Drehgeber-Wert speichern
      Set F_hgb                                   'Set HGB
   End If

Drs_end:
Return

'Slave-Adressen ermitteln
'------------------------
A_440_upadr:
'
Up_adr:
'Adress-Bereich: &H22 - &H6E = 34 - 110
'EEprom: Startadresse
   Hz02 = 1                                       'Adresse im EEprom
   Hz03 = 0                                       'Zhler: Slave-Anzahl
   Hz04 = 0                                       'Zhler: Array

   Call Twi_send(&H00 , 252)                      'cq, 252: LED aus
   For Hz01 = &H22 To &H6F Step 2                 'Adressbereich durchlaufen
      I2cstart : Waitms 100                       'Senden: Start
      I2cwbyte Hz01 : Waitms 100                  'Senden: Adresse
      Incr Hz02                                   'EEprom-Startadresse: 002
      Incr Hz04                                   'Array -Startzelle .: 001
      If Err = 0 Then                             'Adressierung mglich
         Call Lcd_anz( "Slave: " + Str(hz01) + " &H" + Hex(hz01) , "" , 0)
         Wait 1
         Slv_wert(hz04) = Hz01                    'Array beschreiben
'Slave-Adresse in das EEprom schreiben
         Incr Hz03                                'Slave-Anzahl erhhen
         Writeeeprom Hz01 , Hz02                  'EEprom: Adresse [var, adr]
      Else
         Call Lcd_anz( "ADR: " + Str(hz01) + " Error" , "" , 0)
         Wait 1
         Writeeeprom K_epr , Hz02
      End If
      I2cstop : Waitms 100                        'Bus freigeben
   Next Hz01

   Writeeeprom Hz03 , 1                           'EEprom: Anzahl [var, adr= 1]
   K_slvmax = Hz03
   Call Lcd_anz( "Stopp: ADR-Scan" , "Slave-Anzahl: " + Str(k_slvmax) , 0)
   Wait 5

Return

'DCF77-Auswertung
'----------------
'Das UP wird durch den DCF-Timer automatisch jede Sekunde einmal aufgerufen.
'$Time und Date$ werden durch die Funkuhr aktualisiert.
A_450_updcf:
'
Sectic:
   Reset Watchdog
   Set F_dcf                                      'Flag fr LED-Taktanzeige

   If Z_sec < 59 Then
      Incr Z_sec
   Else
      Z_sec = 0
   End If

   If Z_sec = 0 Or Z_sec = 30 Then                'alle 30 s Programmwechsel im
      Set F_apw                                   'Automatikbetrieb
   End If

Return

'-------------------------------------------------------------------------------
'50. === U N T E R P R O G R A M M - C A L L ===================================
'-------------------------------------------------------------------------------
'
A_510_calllcd:
'
'LCD-Ausgabe
'-----------
Sub Lcd_anz(lcd_z1 As String , Lcd_z2 As String , Lcd_time As Byte)
'Prfen, ob sich die Anzeige gendert hat
   If Lcd_z1 <> Lcd_alt1 Or Lcd_z2 <> Lcd_alt2 Then
'LCD-HGB bei einer nderung einschalten (Zeitbegrenzung bei Timer 0)
      If Z_prg <> 1 Then                          'nicht bei der Zeitanzeige
         Set F_hgb                                'Set Flag: LCD_HGB
         Reset F_hgb_dcf                          'Reset bei anderem Programm
      Else
         If F_hgb_dcf = 0 Then                    'nur 1x HGB einschalten
            Set F_hgb_dcf
            Set F_hgb
         End If
      End If

      Cls
      Locate 1 , 1 : Lcd Lcd_z1                   'Ausgabe Zeile 1
      Locate 2 , 1 : Lcd Lcd_z2                   'Ausgabe Zeile 2
      Lcd_alt1 = Lcd_z1
      Lcd_alt2 = Lcd_z2
'LCD-HGB
      T_lcd = 1000 * Lcd_time                     'Timer-Wert (n x 1 ms = y s)
   End If

End Sub

A_520_calltwi:
'
'Daten an den Slave senden
'-------------------------
Sub Twi_send(aby As Byte , Dby As Byte )
   I2csend Aby , Dby                              'Senden: Adresse, Datenbyte
'V 2.0.13 - Der Befehl "I2cstop" ist eigentlich nicht erforderlich; nach dem
'           Eintrag treten aber kaum noch Fehler bei der Datenbertragung auf.
'           "Waitms n" reduziert nochmals die Fehlerquote (getestet bei einem
'           Bustakt von 50 kHz).
   Waitms 5
   I2cstop
   Waitms 10

End Sub

A_530_callwait:
'
'Pause mit Abfrage einer Tastenbettigung
'----------------------------------------
Sub Wait_tas(w_t As Byte , W_b As Byte)
   If F_aut = 1 Then
      Prg_a = 0

      For Hz_wait = 1 To W_t                      'Zhler fr Basiszeit [ms]
'V 2.013 - Programmfortschaltung im Automatikbetrieb
         If F_tw3 = 1 Then
            Reset F_tw3
            Incr Prg
            If Prg = 8 Then Prg = 28
         End If

         If F_tas = 1 Then                        'Routine Timer 0
            Reset F_tas
            Exit For                              'Abbruch der Wartezeit
         End If
         Waitms W_b
      Next Hz_wait
   End If

End Sub

'-------------------------------------------------------------------------------
'60. === I N I T I A L I S I E R U N G   D E R   S L A V E S ===================
'-------------------------------------------------------------------------------
'
'Steuerung
'~~~~~~~~~
'Die folgenden Werte steuern nach der bertragung zum Slave dessen Routinen.

'LED-Farbe
'~~~~~~~~~
' 1: 0001 - rot
' 2: 0010 - grn
' 3: 0011 - blau
' 4: 0100 - gelb
' 5: 0101 - cyan
' 6: 0110 - magenta
' 7: 0111 - wei
' 8: 1000 - (zufllig)

'Modus
'~~~~~
'Folgende Modi sind in der Software der Slaves implementiert:
'010 - Flackerlicht (Farbe "gelb" wird im Slave initialisiert)
'011 - Grundhelligkeit mit Blitz
'012 - kontinuierlicher Helligkeitswechsel

'Steuerwert
'~~~~~~~~~~
'250 - Slave LED-Test
'251 - Slave LED ein
'252 - Slave LED aus
'253 - Slave Modus starten
'254 - Slave Modus beenden
'255 - Slave synchronisieren (SYN); => Reset Slave-Timer

A_610_slave:
'
Slv_prg:
'V 2.0.08
   Reset F_tas                                    'Reset Flag Tasteneingabe

   Select Case Prg
'Farbe
'~~~~~
      Case 1 To 8                                 'Case n <Prg>
         Restore Dat_0108
         M_seq = 1

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

'Modus
'~~~~~
      Case 10                                     'Case n <Prg>
         Restore Dat_10
         M_seq = 1
      Case 11                                     'Case n <Prg>
         Restore Dat_11
         M_seq = 1
      Case 12                                     'Case n <Prg>
         Restore Dat_12
         M_seq = 1
      Case 13                                     'Case n <Prg>
'Restore in der Routine, weil nacheinander fr alle Slaves erforderlich.
         M_seq = 2

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

'Bereich fr weitere Aufrufe im Automatikbetrieb
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'cq: Farbe, Blitzen
      Case 28 To 34                               'Case n <Prg>
         Restore Dat_28
         Led_f = Prg - 27                         'ergibt Farbe rt -> ws
         M_seq = 1
'cq: Farbe, Helligkeitswechsel
      Case 35 To 41                               'Case n <Prg>
         Restore Dat_35
         Led_f = Prg - 34                         'ergibt Farbe rt -> ws
         M_seq = 1
'Farbreihe vorwrts
      Case 42 To 48                               'Case n <Prg>
         Restore Dat_20
         Led_f = Prg - 41                         'ergibt Farbe rt -> ws
         M_seq = 2
'Farbspiel bunt
      Case 49 To 55                               'Case n <Prg>
'        Restore -
         M_seq = 10
'Farbreihe rckwrts
      Case 56 To 62                               'Case n <Prg>
         Restore Dat_20
         Led_f = Prg - 55                         'ergibt Farbe rt -> ws
         M_seq = 3
'Wechsel aller LED mit 2 Farben
      Case 63 To 69                               'Case n <Prg>
'        Restore -
         M_seq = 11
'Wechsel aller LED mit 3 Farben
      Case 70 To 76                               'Case n <Prg>
'        Restore -
         M_seq = 12
'Lauflicht
      Case 77 To 83                               'Case n <Prg>
'        Restore -
         M_seq = 13

      Case 250 To 255                             'Case n <Prg>
         M_seq = 0
         Call Twi_send(&H00 , Prg)                'cq
'Ende
      Case Else
         Prg = 0
         Goto Slv_ret
   End Select

'LCD: Anzeige des aktuellen Programms
   If F_aut = 1 Then
      Locate 2 , 1
      Lcd Prg ; "/" ; Led_f
   End If

'Daten senden
'~~~~~~~~~~~~
   Select Case M_seq

'Farbe fr LED
      Case 1                                      'Case n <M_seq>
'V 2.0.13 - max. Anzeigedauer bei Bettigung DRS ausschalten
         Reset F_drs

         Read Rz                                  'Anzahl der Steuerbytes
         For Hz02 = 1 To Rz
            Read Rb                               'Datenbyte lesen
            If Hz02 = 1 Then
               Select Case Prg                    'Option LED-Farbe
                  Case 1 To 8
                     Rb = Prg
                  Case 12
                     Rb = 1 + Rnd(7)              'Werte: 1+ [0 bis 6] => 1-7
                  Case 28 To 42
                     Rb = Led_f
               End Select
            End If
            Call Twi_send(&H00 , Rb)              'cq
         Next Hz02
'Wartezeit mit Reaktion auf Tastenbettigung
        Call Wait_tas(200 , 100)                  '200* 100 ms= 20 s

'Reihenfolge: vorwrts
      Case 2                                      'Case n <M_seq>
'An alle vorhandene Slaves wird nacheinander der Code bertragen
         For Hz01 = 1 To K_slvmax
'Wartezeit mit Reaktion auf Tastenbettigung
            Call Wait_tas(50 , 5)                 '50* 5 ms= 250 ms
            Restore Dat_20
            Read Rz                               'Anzahl Codebyte
            For Hz02 = 1 To Rz
               Select Case Hz02
                  Case 1 To 2                     '1./2. Byte senden
                     Read Rb
                     If Hz02 = 1 And Prg > 41 And Prg < 49 Then
                        Rb = Led_f                'Automatik: Farbwechsel
                     End If
                     Call Twi_send(slv_wert(hz01) , Rb)
                  Case Else
                     Read Rp                      '3. Byte senden
                     Waitms Rp
               End Select
            Next Hz02
'bei Handbetrieb Abbruch mit Taste 1
            If F_tw1 = 1 Then
               Reset F_tw3
               Exit For
            End If
         Next Hz01
'Wartezeit mit Reaktion auf Tastenbettigung
         Call Wait_tas(50 , 5)                    '50* 5 ms= 250 ms

'Reihenfolge: rckwrts
      Case 3                                      'Case n <M_seq>
'An alle vorhandene Slaves wird nacheinander der Code bertragen
'Alle LED einschalten
         Call Twi_send(&H00 , Led_f)              'cq,   x: Farbe n
         Call Twi_send(&H00 , 251)                'cq, 251: LED Ein
'Reihenfolge: rckwrts
         For Hz01 = K_slvmax To 1 Step -1
'Wartezeit mit Reaktion auf Tastenbettigung
            Call Wait_tas(50 , 5)                 '50* 5 ms= 250 ms
            Call Twi_send(slv_wert(hz01) , 252)   'ADR, 252: LED aus
'bei Handbetrieb Abbruch mit Taste 1
            If F_tw1 = 1 Then
               Reset F_tw1
               Exit For
            End If
         Next Hz01
'Wartezeit mit Reaktion auf Tastenbettigung
         Call Wait_tas(50 , 5)                    '50* 5 ms= 250 ms

'alle LED mit Zufallsfarbe
      Case 10                                     'Case n <M_seq>
'Alle LED leuchten mit einer zuflligen Farbe, die sich zyklisch ndert.
'Zu Beginn werden alle LED mit einer zuflligen Farbe initialisiert.
         For Hz_int = 1 To K_slvmax
            Rb = 1 + Rnd(7)                       'Werte: 1+ [0 bis 6]   => 1-7
            Call Twi_send(slv_wert(hz_int) , Rb)  'ADR,   x: Farbe n
            Call Twi_send(slv_wert(hz_int) , 251) 'ADR, 251: LED ein
         Next Hz_int

         For Hz_int = 1 To 120                    'Zyklusdauer: 120 s
            For Hz01 = 0 To Rnd(13)
               Waitms 10
            Next Hz01

            Hz03 = K_slvmax - 1                   'Anzahl Slaves
            Hz02 = 1 + Rnd(hz03)                  'Werte: 1+ [0 bis n-1] => 1-n
            Rb = 1 + Rnd(7)                       'Werte: 1+ [0 bis 6]   => 1-7
            Call Twi_send(slv_wert(hz02) , Rb)    'ADR,   x: Farbe n
            Call Twi_send(slv_wert(hz02) , 251)   'ADR, 251: LED ein
'Wartezeit mit Reaktion auf Tastenbettigung
            Call Wait_tas(100 , 10)               '100* 10 ms= 1 s
'bei Handbetrieb Abbruch mit Taste 1
            If F_tw1 = 1 Then
               Reset F_tw1
               Exit For
            End If
         Next Hz_int

'alle LED mit zwei Farben
      Case 11                                     'Case n <M_seq>
         Hz01 = 1 + Rnd(7)
Slv_11:                                           'neue Farbe, wenn gleich
         Hz02 = 1 + Rnd(7)
         If Hz02 = Hz01 Then Goto Slv_11

         For Hz_int = 1 To K_slvmax
            If F_led = 0 Then
               Call Twi_send(slv_wert(hz_int) , Hz01)       'ADR, x: Farbe n
            Else
               Call Twi_send(slv_wert(hz_int) , Hz02)       'ADR, x: Farbe n
            End If
            Toggle F_led
         Next Hz_int
'LED einschalten (cq)
         Call Twi_send(&H00 , 251)                'cq, 251: LED ein
'Wartezeit mit Reaktion auf Tastenbettigung
         Call Wait_tas(200 , 10)                  '200* 10 ms= 2 s

'alle LED mit drei Farben
      Case 12                                     'case n <M_seq>
         Hz01 = 1 + Rnd(7)                        'Zufallsfarbe 1
Slv_121:                                          'neue Farbe, wenn gleich
         Hz02 = 1 + Rnd(7)                        'Zufallsfarbe 2
         If Hz02 = Hz01 Then Goto Slv_121
Slv_122:                                          'neue Farbe, wenn gleich
         Hz03 = 1 + Rnd(7)                        'Zufallsfarbe 3
         If Hz03 = Hz01 Or Hz03 = Hz02 Then Goto Slv_122

         For Hz_int = 1 To K_slvmax Step 3
            Rb = Hz_int
               Call Twi_send(slv_wert(rb) , Hz01) 'ADR 1, x: Farbe 1
            Incr Rb
            If Rb <= K_slvmax Then
               Call Twi_send(slv_wert(rb) , Hz02) 'ADR 2, x: Farbe 2
            End If
            Incr Rb
            If Rb <= K_slvmax Then
               Call Twi_send(slv_wert(rb) , Hz03) 'ADR 3, x: Farbe 3
            End If
         Next Hz_int
'LED einschalten (cq)
         Call Twi_send(&H00 , 251)                'cq, 251: LED ein
'Wartezeit mit Reaktion auf Tastenbettigung
         Call Wait_tas(200 , 10)                  '200* 10 ms= 2 s

'Lauflicht
      Case 13                                     'Case n <M_seq>
'fr alle LED wird eine Farbe eingestellt
         Hz01 = 1 + Rnd(7)                        'Zufallsfarbe
         Call Twi_send(&H00 , Hz01)               'cq, x: Farbe n
'Anzahl der leuchtenden LED
         Hz04 = 2 + Rnd(5)                        'Zufallsanzahl
         Hz01 = K_slvmax + Hz04                   'alle Slave-Adressen

         For Hz02 = 1 To Hz01
            Hz_int = Hz02
            For Hz03 = 1 To Hz04
               If Hz_int > 0 And Hz_int <= K_slvmax Then
                  Call Twi_send(slv_wert(hz_int) , 251)       'ADR, 251: LED ein
               End If
               Decr Hz_int
            Next Hz03
'Ausschalten erst, wenn die erste Gruppe vollstndig leuchtet
            If Hz02 > Hz04 Then
               Decr Hz_int
               Call Twi_send(slv_wert(hz_int) , 252)       'ADR, 252: LED aus
            End If
'Wartezeit mit Reaktion auf Tastenbettigung
            Call Wait_tas(10 , 10)                '10* 10 ms= 100 ms
'bei Handbetrieb Abbruch mit Taste 1
            If F_tw1 = 1 Then
               Reset F_tw1
               Exit For
            End If
         Next Hz02
         Call Twi_send(slv_wert(hz02) , 252)      'ADR, 252: LED aus

   End Select

Slv_ret:
   M_seq = 0

Return

'-------------------------------------------------------------------------------
'70. === F U N K T I O N =======================================================
'-------------------------------------------------------------------------------

'-------------------------------------------------------------------------------
'80. === D A T E N B E R E I C H ===============================================
'-------------------------------------------------------------------------------
'
A_800_daten:
'
'Text (Mx: Modus n)
Dat_far:
Data "rot" , "gruen" , "blau" , "gelb" , "cyan" , "magenta" , "weiss" , "Zufall"

Dat_0108:                                         'Anzahl: 2
   Data 2 , 0 , 251                               'Farbe, LED ein
Dat_10:                                           'Anzahl: 2
   Data 2 , 10 , 253                              'M10, M10 ein
Dat_11:                                           'Anzahl: 2
   Data 2 , 11 , 253                              'M11, M11 ein
Dat_12:                                           'Anzahl: 4
   Data 4 , 0 , 251 , 12 , 253                    'Farbe, LED ein, M12, M12 ein

'Slaves nacheinander (Reihe) aufleuchten lassen (1: Farbe rot, 100: Pause [ms])
Dat_20:                                           'Anzahl: 3
   Data 3 , 1 , 251 , 100                         'Farbe, LED ein, Pause

'Automatik
'~~~~~~~~~
'LED-Farbwechsel mit Blitzen
Dat_28:                                           'Anzahl: 4
   Data 4 , 0 , 251 , 11 , 253                    'Farbe, LED ein, M11, M11 ein
Dat_35:                                           'Anzahl: 4
   Data 4 , 0 , 251 , 12 , 253                    'Farbe, LED ein, M12, M12 ein

'Ende Datenbereich

'###############################################################################


'Folgender Programmcode soll als Einstieg zum Einsatz einer Micro-SCcard dienen.

A_900_sdcard:
'
'-------------------------------------------------------------------------------
'90. === S D C A R D  ==========================================================
'-------------------------------------------------------------------------------
'
'Code zur Anbindung einer Speicherkarte
'--------------------------------------
'Verwendet wird eine Micro-SDcard.
'(
Z_SDCard:
'Initialisierung SDCard
'Auswahl
'0: Hardware-SPI
'1: Software-SPI
   Const Cmmc_soft = 1                            'verlangt die Bibliothek
#if Cmmc_soft = 0
'Pin fr Chip-Select
   Config Pinb.4 = Output                         'PB4 fr CS
   Mmc_cs Alias Portb.4
   Set Mmc_cs                                     'Set CS

'SS-Pin der HW-SPI
   Config Pinb.4 = Output                         'PB4 fr SPI_SS
   Spi_ss Alias Portb.4
   Set Spi_ss                                     'SPI als Master

   Config Spi = Hard , Interrupt = Off , Data Order = Msb , Master = Yes , _
     Polarity = High , Phase = 1 , Clockrate = 4 , Noss = 1
   Spiinit                                        'SPI initialisieren
#else
'Achtung: Alias (x) muss dem Port (.x) entsprechen (MOSI, MISO, SCK)
'Pin fr Chip-Select
   Config Pind.4 = Output
   Mmc_cs Alias Portd.4
   Set Mmc_cs
'MOSI-Pin der SW-SPI
   Config Pind.5 = Output
   Set Pind.5
   Mmc_portmosi Alias Portd
   Bmmc_mosi Alias 5
'MISO-Pin der SW-SPI
   Config Pind.6 = Input
   Mmc_portmiso Alias Pind
   Bmmc_miso Alias 6
'SCK-Pin der SW-SPI
   Config Pind.7 = Output
   Set Pind.7
   Mmc_portsck Alias Portd
   Bmmc_sck Alias 7
#endif

'Fehler
'-E: Error response Byte ...
'-n: no Data response Byte ...
'-r: return code for DriveGetIdentity
   Const Cperrdrivereset = 225                    'E ... at Reset command
   Const Cperrdriveinit = 226                     'E ... at Init Command
   Const Cperrdrivereadcommand = 227              'E ... at Read Command
   Const Cperrdrivewritecommand = 228             'E ... at Write Command
   Const Cperrdrivereadresponse = 229             'N ... from MMC at Read
   Const Cperrdrivewriteresponse = 230            'N ... from MMC at Write
   Const Cperrdrive = 231
   Const Cperrdrivenotsupported = 232             'r ..., not supported yet

   Waitms 1                                       'Wartezeit vor Initialisierung
   Dim Gbdriveerror As Byte                       'Treiber Fehler
   Dim Gbdriveerrorreg As Byte                    'Treiber Fehler-Register
   Dim Gbdrivestatusreg As Byte                   'Treiber Status-Register
   Dim Gbdrivedebug As Byte

   $lib "MMC.LIB"                                 'Bibliothek MMC/SD Card
   $external _mmc                                 'Einschluss ASM-Code
   Gbdriveerror = Driveinit()                     'Initialisierung MMC/SD Card

Cls : Lcd "Init " + Str(gbdriveerror) : Wait 3

   Dim Hz_sector As Word                          'Hilfszhler: Sector
   Dim Abuffer(512)as Byte                        'Array: Sector mit 512 Byte
   Dim Wsrampointer As Word                       'Zeiger auf Speicherzelle
   Dim Sectornr As Long                           'Sector-Nummer
   Wsrampointer = Varptr(abuffer(1))

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

'Sector lesen
   Sectornr = 7624                                'Sector (physisch)
   Gbdriveerror = Drivereadsector(wsrampointer , Sectornr )     'Zeitdauer: 8 ms
   Waitms 50

   Cls : Lcd "Read: " + Str(gbdriveerror) : Wait 3

   For Hz_sector = 1 To 512
      Cls : Lcd Hz_sector ; "-" ; Abuffer(hz_sector) : Waitms 1000
   Next Hz_sector
   Wait 5
')

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

'Ende
