'Programm ......... : LAD_TWI-Slave (Adresse lt. Label A_124_adr)
'
'Programminhalt ... : - TWI-Buskopplung eines LAD-Moduls (RGB-LED)
'                     - RGB-LED-Steuerung mit pulsweitenmodulierten Einstellung
'                       der Helligkeit
'                     - Adressierung bei Programmstart
'
'Programmierer .... : Ingolf Bauer (ingolf.bauer@nexgo.de)
'Version .......... : V 1.1.02
'Datum ............ : 28.12.2017
'
'-------------------------------------------------------------------------------
'
A_010_information:
'
'TWI-Datenbyte
'-------------
'Initialisierung
' 001 Farbeinstellung rot
' 002 Farbeinstellung grn
' 003 Farbeinstellung blau
' 004 Farbeinstellung gelb
' 005 Farbeinstellung cyan
' 006 Farbeinstellung magenta
' 007 Farbeinstellung wei
' 008 Farbeinstellung <Zufallsfarbe)
'
'Modus
' 010 Flackerlicht (gelb)
' 011 Farbe mit Grundhelligkeit und Zufallsblitz
' 012 Farbe mit rythmischem Helligkeitswechsel
'
'Sequenz
' 250 Aktivierung LED-Farbtest
' 251 LED einschalten
' 252 LED ausschalten
' 253 Modus starten
' 254 Modus beenden
' 255 Synchronisation (Reset Timer)
'
'-------------------------------------------------------------------------------
'
'Simulator-Einstellmglichkeit
' Der folgende Befehl teilt dem Compiler mit, keine wzschleifen bei den
' Befehlen "Wait" und "Waitms" zu beachten. Dies ermglicht eine schnellere
' Simulation.
'
'$sim                                              'Wartezeit aus
'
' Achtung!
' Diese Zeile unbedingt auskommentieren, bevor der AVR programmiert wird, da
' sonst der AVR nicht korrekt arbeitet.
' Die Aktivierung ist nur fr den Simulator vorgesehen.
'
'-------------------------------------------------------------------------------
'
'Information
'-----------
'Initialisierung
' Nach einer neuen Programmierung ist es unbedingt erforderlich, dass alle am
' TWI-Bus angeschlossenen AVR abgeschaltet und wieder neu eingeschaltet werden.
'
'-------------------------------------------------------------------------------
A_020_historie:
'
'Versionsbersicht
'-----------------
'
'
'
'
'
'
'
'
'
'V 1.1.02  28.12.2017 - nderung .... : - Marker fr Adresse eingefgt
'                                       - Kommentar korrigiert
'V 1.1.01  26.03.2017 - Korrektur ... : - Programmfehler behoben
'                                       - Anzahl der Datenbytes: 1
'V 1.0.04  05.03.2016 - Korrektur ... : - diverse Fehlerkorrekturen
'V 1.0.02  12.01.2016 - Erweiterung . : - Implementierung (Funktion, Modus)
'V 1.0.01  10.01.2016 - Information . : - Beginn der Softwareentwicklung
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'Programmspeicher des MCs (Programm-Flashspeicher): V 1.1.02 - 36%
'
'-------------------------------------------------------------------------------
'
'Informationen zur Software
'--------------------------
' Inhalt                                                                | Zeile
'-------------------------------------------------------------------------------
' 01. Allgemeine Informationen ........................................ |  132
' 02. Abkrzungsbersicht ............................................. |  146
' 03. Compilereinstellungen ........................................... |  152
' 04. Funktionsbeschreibung ........................................... |  176
' 05. Interrupt ....................................................... |  208
' 06. Einstellung der FUSE- und/oder LOCK-Bits ........................ |  212
' 07. Anschluschema ATtiny85 (baugleich mit ATtiny 25/45) ............ |  220
' 08. I/O-Ports ....................................................... |  233
' 09. Steckverbinder .................................................. |  276
' 10. Allgemein ....................................................... |  281
'      Schaltkreis .................................................... |  285
'      Watchdog ....................................................... |  294
'      Compilereinstellung ............................................ |  298
' 12. Fuse- und Lockbits .............................................. |  306
' 13. Deklaration ..................................................... |  319
' 14. Konfiguration ................................................... |  377
'      I/O-Port ....................................................... |  379
'      Timer .......................................................... |  394
'      TWI-Datentransfer .............................................. |  428
'      Interrupt ...................................................... |  443
' 15. Alias ........................................................... |  454
'      Ausgang (digital) .............................................. |  456
' 16. Initialisierung ................................................. |  465
'      Konstanten ..................................................... |  469
'      Lebenszeichen .................................................. |  481
' 20. Programm ........................................................ |  502
' 30. Interruptroutine ................................................ |  702
'      Timer 0 ........................................................ |  706
'      PWM-Steuerung der LED (Helligkeit) ............................. |  764
' 40. Programm ........................................................ |  782
' 50. Unterprogramm-Call .............................................. |  786
'      LED-Farbe whlen ............................................... |  790
'      Wartezeit mit Modusabbruch bei Datenempfang .................... |  825
'      TWI-Bus ........................................................ |  844
' 70. Funktion ........................................................ |  907
' 80. Datenbereich .................................................... |  911
'-------------------------------------------------------------------------------
'
'01. Allgemeine Informationen
'----------------------------
'Taktfrequenz
'- - - - - - -
' Fr die Taktfrequenz wird der interne Oszillator mit 20 MHz (PLL) verwendet.
'
'Programmierung
'- - - - - - - -
' Die Programmierung des MCs kann extern oder ber die vorhandene Schnittstelle
' erfolgen.
' Zur Programmierung wurde das Modul "USBasp" verwendet, welches ber eine
' USB-Schnittstelle angeschlossen wird. Es kann direkt ber eine, in BASCOM
' integrierte Einstellung angesprochen werden.
'
'02. Abkrzungsbersicht
'-----------------------
' PWM ... : Pulsweitenmodulation
' TWI ... : Two-wire Serial Modus        (entspricht technisch dem I2C-Bus)
' USI ... : Universal Serial Interface   (meist bei ATtiny vorhanden)
'
'03. Compilereinstellungen
'-------------------------
' Hinweise fr ATmega8/ATmega16 mit 1024 Byte SRAM
' Sie gelten ggf. auch fr den hier eingesetzten Mikrocontroller.
' Quelle:
' //halvar.at/elektronik/kleiner_bascom_avr_kurs/speicher_hwstack_swstack_frame/
'
' HW-Stack .. - Empfehlung: 40 Byte bei INT, sonst 10 Byte ausreichend
'  32 Byte fr INT + je GOSUB 2 Byte, wenn bei INT ein GOSUB erfolgt
'  (bei Verschachtelung ... + n x 2 Byte)
'
' SW-Stack .. - Empfehlung: 32 Byte (recht komplexe Aufrufe werden abgedeckt)
'   2 Byte fr jede bergebene Variable einer GOSUB-Routine + 2 Byte fr jede
'   LOCAL-Variable innerhalb einer GOSUB
'   (bei Verschachtelung ... n x 2 Byte)
'
' Frame-Space - Empfehlung: 60 Byte sollten alles abdecken
'                           (bei wenig Speicher verkleinern)
'  Beispiel:
'   2 x BYTE        =  2 Byte (erster und zweiter Parameter)
'   1 x WORD        =  2 Byte (dritte Parameter)
'   1 x STRING * 10 = 11 Byte (inkl. Abschlussbyte; vierter Parameter)
'   3 x BYTE        =  3 Byte (lokale Variablen)
'
'04. Funktionsbeschreibung
'-------------------------
'Allgemeines
'- - - - - -
'Stromversorgung
'Sie erfolgt im normalen Betrieb ber die Schnittstelle, ber die auch der
'TWI-Bus angeschlossen wird und muss stabilisiert sein sowie einen Wert von +5 V
'haben.
'Es ist auf dem Modul keine Sicherung bei Kurzschluss oder Verpolung vorhanden.
'
'Anschluss
'- - - - -
'Fr den Anschluss einer RGB-LED werden drei Ausgabekanle eingesetzt.
'Beim Programmstart blinkt die LED dreimal kurz hintereinander (grn).
'
'Adressierung
'- - - - - -
'Bei Modulen, die ber ein Bussystem verbunden sind, muss jeder Teilnehmer eine
'eindeutige Adresse erhalten.
'Diese Basisadresse kann im Programm verndert werden (Label: A_124_adr).
'
'Adresse | R/W | Beschreibung                            | &Hxx  | &DEZ
'---------------------------------------------------------------------------
'0000000 |  0  | General Call Adresse                    |    00 |         0
'0000000 |  1  | Startbyte                               |    01 |         1
'0000001 |  X  | CBUS Adresse                            | 02-03 |   2 -   3
'0000010 |  X  | Reserviert fr ein anderes Busformat    | 04-05 |   4 -   5
'0000011 |  X  | Fr zuknftige Erweiterungen reserviert | 06-07 |   6 -   7
'00001XX |  X  | Fr zuknftige Erweiterungen reserviert | 08-0F |   8 -  15
'11110XX |  X  | 10-Bit Adressierung                     | 70-F7 | 112 - 247
'11111XX |  X  | Fr Zuknftige Erweiterungen Reserviert | F8-FF | 248 - 255
'
'05. Interrupt
'-------------
' Timer 0:  8 Bit, Interrupt bei berlauf (PWM-Steuerung)
'
'06. Einstellung der FUSE- und/oder LOCK-Bits
'--------------------------------------------
' Einstellungshinweise zu den Fuse- und Lockbits finden sich unter Punkt 10.
'
' Anpassungen sind bei der Verwendung des Brenners "USBasp" mit einem internen
' Brennprogramm von BASCOM mglich.
' z. B. Lschung des EEproms zusammen mit dem Brennen des Programms
'
'07. Anschluschema ATtiny85 (baugleich mit ATtiny 25/45)
'--------------------------------------------------------
A_030_avr:
'                   --------------|_|-------------
' [~]  /Reset ---> | 1 [/Reset]            [VCC] 8| ==== P5V
' [0]   RGB_1 <--- | 2 [PB3]          [SCK, SCL] 7| ---> SCK, TWI     [SPI, #]
' [0]   RGB_2 <--- | 3 [PB4]         [MISO, PB1] 6| ---> MISO, RGB_3  [SPI, 0]
'         GND ==== | 4 [GND]         [MOSI, SDA] 5| <--- MOSI, TWI    [SPI, #]
'                   ------------------------------
'
' Werte in []: Einstellungen nach dem Programmstart
'  ~: Impuls   0: aus   1: ein   #: Bussystem
'
'08. I/O-Ports
'-------------
' /Reset  : Resetfunktion
' LED_    : Anschluss RGB-LED
' SDA     : Busanschluss (Daten)
' SLC     : Busanschluss (Takt)
'
' P5V     : Stromversorgung P5 V
' GND     : Stromversorgung Masse
'
' MISO    : Programmieranschlu (SPI)
' MOSI    : Programmieranschlu (SPI)
' SCK     : Programmieranschlu (SPI)
'
' _<n>    : Anschluss RGB-LED (n: 1-3 fr die Farben rot, grn, blau)
'
'           Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' -   : SV  P5V ........ | ............. | Stromversorgung .............. |  8
' -   : SV  GND ........ | ............. | Stromversorgung .............. |  4
'
'Port B     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DE  SPI, TWI ... | MOSI, SDA ... | Programmierung, Systembus .... |  5
' - .1: DA  SPI, RGB_3 . | MISO, LED ... | Programmierung, RGB-LED blau . |  6
' - .2: DA  SPI, TWI ... | SCK, SCL .... | Programmierung, Systembus .... |  7
' - .3: DA  RGB_1 ...... | LED ......... | RGB-LED rot .................. |  2
' - .4: DA  RGB_2 ...... | LED ......... | RGB-LED grn ................. |  3
' - .5: DE  /RESET ..... | ............. | Programmierung, /Reset ....... |  1
' - nc: ... ............ | ............. | .............................. | nc
' - nc: ... ............ | ............. | .............................. | nc
'
'Erluterung
'- - - - - -
' DA      : digitaler Ausgang
' DE      : digitaler Eingang
' PRG     : Programmierung
' RGB-LED : LED
' SPI     : Programmierung
' SV      : Stromversorgung
' TWI     : Systembus
' USI     : universelle Schnittstelle
'
'09. Steckverbinder
'------------------
' X1 : Programmer                                 Steckverbinder (6-polig)
'
'-------------------------------------------------------------------------------
'10. === A L L G E M E I N =====================================================
'-------------------------------------------------------------------------------
A_110_schaltkreis:
'
'Schaltkreis
'-----------
   $regfile = "attiny85.dat"                      'Prozessor ....... : ATtiny 85
   $crystal = 20000000                            'Frequenz ........ : 20,0 MHz

'Taktfrequenz
' PLL-Frequenz mi 20 MHz                          'Einstellung FUSE
' interner RC  mit 8 MHz                          'Einstellung FUSE

'Watchdog
'--------
   Config Watchdog = 2048                         'Reset nach Timeout 2048 ms

'Compilereinstellung
'-------------------                              'bisher
   $hwstack = 40                                  '--
   $swstack = 32                                  '--
   $framesize = 60                                '--

A_120_fuse:
'
'12. Fuse- und Lockbits
'----------------------
'USBasp: Werte generiert mit "Write PRG"
'$PROG &HFF,&HC1,&HDF,&HFF                        'PLL-Frequenz: 20 MHz
' FF: 11
' C1: 1 1 000001
' DF: 1 1 0 1 1 111
' FF: 1
'
'-----------------------------------------------------------------------------
'
A_130_deklarartion:
'
'13. Deklaration
'---------------
'Bibliothek
'Betriebsart: USI
' Die Bibliothek ist ein Add-On zu BASCOM (Funktion als Slave am TWI-Bus durch
' Verwendung der USI-Schnittstelle des ATtiny 85).
'
   $lib "I2C_USI_SLAVE.LIB"

'Subrotine (Call)
'-Einstellung LED-Farbe
   Declare Sub Led_col(byval V_rgb As Byte)       'RGB-Einstellung (LED-Farbe)
'-Wartezeit
   Declare Sub Wz(byval V_wz As Word)             'Aufruf Wartezeit

'Feld
   Dim Twi_db As Byte                             'Daten: Twi
   Dim Twi_dba As Byte                            'Daten: alt

'Flag
   Dim F_led As Boolean                           'LED ..... : Ein / Aus
   Dim F_mod As Boolean                           'Modus ... : Ein / Aus
   Dim F_hew As Boolean                           'Richtungswechsel
   Dim F_dul As Boolean                           'PRG-Durchlasuf 1x
   Dim F_rt As Boolean                            'Farbe aktiv: rot
   Dim F_gn As Boolean                            'Farbe aktiv: grn
   Dim F_bl As Boolean                            'Farbe aktiv: blau
   Dim F_db As Boolean                            'Flag: Datenbertragung
   Dim F_st As Boolean                            'Flag: Start
   Dim F_wz As Boolean                            'Flag: Wartezeit abgebrochen

'Konstante
   Dim K_mi As Byte                               'Helligkeit: minimal
   Dim K_mh As Byte                               'Helligkeit: mittel
   Dim K_mx As Byte                               'Helligkeit: maximal

'Variable
   Dim Pwm_rt As Byte                             'Helligkeit: LED_rot
   Dim Pwm_gn As Byte                             'Helligkeit: LED_grn
   Dim Pwm_bl As Byte                             'Helligkeit: LED_blau
   Dim V_ini As Byte                              'Initialisierung
   Dim V_mod As Byte                              'Modus
   Dim V_seq As Byte                              'Sequenz
   Dim V_rnd As Word                              'Zufallsvariable
   Dim V_wz As Integer                            'Variable: Wartezeit

'Timer
   Dim T0_tve As Byte                             'Voreinstellwert: T0

'Zhler
   Dim Hz01 As Byte                               'Hilfszhler 01
   Dim Z_wz As Integer                            'Zhler: Wartezeit

   Dim Z_t0 As Byte                               'Zeit T0
   Dim Z_hew As Byte                              'Helligkeit
'
'-------------------------------------------------------------------------------
'
'14. Konfiguration
'-----------------
'I/O-Port
'--------
A_141_port:
'
'=Port B
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
' LED-Ports: 1, 3, 4
   Ddrb = &B0001_1010                             'PB0,2,5-7 Ein, PB1,3,4 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portb = &B1111_1111                            'PB0-7 aktiv
'
'-------------------------------------------------------------------------------
'
A_142_config:
'
'Timer
'-----
'/// Achtung ///
'Fr den Betrieb mit einer Funkuhr (DCF) ist Timer 1 erforderlich.
'Eine Konfiguration ist nicht erforderlich.

'Normal-Konfiguration
'Die Interrupts von Timer 0 werden beim jeweiligen Timerberlauf ausgelst.
'PSC: 1-8-64-256-1024
'PSC: Prescale; TZW: Timerzhlwert; TVE: Timervoreinstellwert

'       | ATtiny 85 | Funktion
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' 8 bit | Timer 0   | PWM-Steuerung
' 8 bit | Timer 1   | (nicht belegt)
'
'Berechnung der Zeitbasis:
' bei 8 MHz-Quarz: Timer 0/1 (8 bit, Max = 256)
'- - - - - - - - - - - - - - - - - - - - - - -
' Tx = 1/Frequenz * Teiler (PSC) * Voreinstellwert (TZW)
'                    PSC    TZW   T           f          TVE (= Max-TZW)
' T0 = 1/ 8,0 MHz *  256 *  31 =  0,99200 ms;  1.000 Hz; 225
' T0 = 1/20,0 MHz *  256 *  80 =  1,02400 ms;  1.000 Hz; 176
' T0 = 1/20,0 MHz *   64 *  31 =  0,09920 ms; 10.000 Hz; 194
' T0 = 1/20,0 MHz *   64 * 125 =  0,00005 ms; 20.000 Hz; 131

'Timer0 - Konfiguration 20,0 kHz
'Timer fr die LED-Helligkeitssteuerung und der Testroutinen
'-Zeilen-Markierung: #T0
   Config Timer0 = Timer , Prescale = 8           'Konfiguration 20,0 kHz
' Voreinstellung (TVE) laut Berechnung
   T0_tve = 131
   Timer0 = T0_tve                                'Zuweisung

'TWI-Datentransfer
'-----------------
'<Usi> ....... Mode .......... : Twislave (TWI-Slave)
'<Address> ... TWI-Adresse ... : &Hxx     (nur gerade Adressen mglich)
'
'/// Achtung ///
'Interrupts sind global freizugeben, weil USI im Interruptmodus betrieben wird.
'Ggf. sind die beim I2C-Bus reservierten Adressen zu beachten
'I2C-Basisadresse:
A_143_adresse:
'
   Config Usi = Twislave , Address = &H22         'LED 02 - Dezimal 034
'
'-------------------------------------------------------------------------------
'
'Interrupt
'---------
   Enable Timer0                                  'Timer 0: Interrupt
      On Timer0 Int_tim0                          'Timer 0: berlauf

   Enable Interrupts                              'Interrupts: global zulassen
'
'-------------------------------------------------------------------------------
'
A_150_alias:
'
'15. Alias
'---------
'Ausgang (digital)
'-----------------
'Port B
   Led_bl Alias Portb.1                           'LED blau (PRG-Signal "MISO")
   Led_rt Alias Portb.3                           'LED rot
   Led_gn Alias Portb.4                           'LED grn
'
'-------------------------------------------------------------------------------
'
'16. Initialisierung
'-------------------
A_161_konstante:
'
'Konstanten
'----------
'Helligkeiswert
'- minimale Stufe
   K_mi = 5                                       'Helligkeitswert 10%
'- mittlere Stufe
   K_mh = 50                                      'Bereich: 1 < x% < 100
'- hohe Stufe
   K_mx = 95                                      'Helligkeitswert 95%
'
'-------------------------------------------------------------------------------
'
'Lebenszeichen
'-------------
'Blinken beim Programmstart (Programm-Neustart)
'Lebensbit nach Startroutine deaktivieren
   Set F_led                                      'Set LED                                                                    X

   Reset Led_rt : Reset Led_gn : Reset Led_bl     'Reset LED
   Waitms 100                                     'Wartezeit

   For Hz01 = 1 To 6                              'LED_gn (3x)
      Waitms 200                                  'Wartezeit
      If Pwm_gn = 0 Then
         Pwm_gn = K_mh / 2                        'LED_PWM
      Else
         Pwm_gn = 0                               'Reset LED_PWM
      End If
   Next Hz01

   Reset F_led                                    'Reset LED                                                                    X
'
'-------------------------------------------------------------------------------
'20. === P R O G R A M M =======================================================
'-------------------------------------------------------------------------------
A_210_init:
'--
A_220_programm:
'
Do

M_start:
   If Twi_dba <> Twi_db Then                      'Funktionsnderung (Datenbyte)
      Twi_dba = Twi_db
   Else
      Goto M_modus                                'ggf. Modus fortsetzen
   End If

'Die Auswertung des bertragenden Datenbytes erfolgt nur, wenn ein genderter
'Wert bertragen wurde.
   Select Case Twi_dba
      Case 0                                      'ohne Bedeutung
      Case 1 To 8
         Reset F_wz                               'Wartezeit abbrechen
         V_ini = Twi_dba                          'Initialisierung (Farbe)
         Reset F_led                              'LED Aus
      Case 250 To 255
         Reset F_wz                               'Wartezeit abbrechen
         V_seq = Twi_dba                          'Sequenz (Befehl)
      Case Else
         Reset F_wz                               'Wartezeit abbrechen
         V_mod = Twi_dba                          'Modus (Betriebsart)
   End Select

'Initialisierung
'- - - - - - - -
   Select Case V_ini
'Einstellung einer RGB-Farbe (mittlere Helligkeit)
      Case 1 To 8
         Call Led_col(v_ini)
         V_ini = 0                                'Reset Variable <V_ini>
         Reset F_dul                              '1x Durchlauf fr Modus 12
   End Select

'Sequenz
'- - - -
   Select Case V_seq
'LED-Farbtest
      Case 250
         Set F_led                                'Set LED
            Pwm_rt = K_mh : Pwm_gn = 0 : Pwm_bl = 0
            Call Wz(500)                          'Wartezeit mit Abbruchoption
            Pwm_rt = 0 : Pwm_gn = K_mh : Pwm_bl = 0
            Call Wz(500)                          'Wartezeit mit Abbruchoption
            Pwm_rt = 0 : Pwm_gn = 0 : Pwm_bl = K_mh
            Call Wz(500)                          'Wartezeit mit Abbruchoption
         Reset F_led                              'Reset LED

'LED-Aktivierung ein
      Case 251
         Set F_led                                'Set LED

'LED-Aktivierung aus
      Case 252
         Reset F_led                              'Reset LED

'LED-Steuerung (Modus) starten
      Case 253
         Set F_st
         Set F_mod                                'Set Modus
         Set F_led

'LED-Steuerung (Modus) beenden
      Case 254
         Reset F_mod                              'Reset Modus
         Reset F_led                              'Reset LED

'Synchronisation aller Slaves (Reset Timer0 und der Zeit-Zhler)
      Case 255
         Reset F_led                              'Reset LED
         Pwm_rt = 0 : Pwm_gn = 0 : Pwm_bl = 0     'Reset LED_PWM
         Disable Timer0                           'Reset INT_Timer0
            Timer0 = T0_tve                       'Voreinstellung
            Z_t0 = 0                              'Reset Zeit-Zhler
         Enable Timer0                            'SET INT_Timer0
   End Select

'Modus (LED-Steuerung)
'- - - - - - - - - - -
M_modus:
   If F_mod = 1 Then                              'Modus aktiv
'> Case V_mod
      Select Case V_mod                           'Modus

'010 - Flackerlicht gelb (Kerzenbetrieb)
'= = = = = = = = = = = = = = = = = = = =
         Case 10
M010_1:
            V_rnd = Rnd(18) + 5                   'Grundhelligkeit: 5 bis 23

'Helligkeitssteuerung (zufllig)
'- - - - - - - - - - - - - - - -
'Die Werte werden nur im PWM-Zeitfester 95-100 aktualisiert, in der die PWM fr
'die einzelnen LED-Farben deaktiviert ist.
'Farbe 'gelb' wird gemischt (Parameter: 4, abhngig vom LED-Typ )
M010_2:
            If Z_t0 > 95 Then                     'am PWM-Ende umschalten
               Pwm_bl = 0
               Pwm_gn = V_rnd                     'Anteil grn
               Pwm_rt = 4 * Pwm_gn                'Anteil rot
            Else
               Goto M010_2                        'Modus fortfhren
            End If

'Wechselzeitsteuerung (zufllig)
            V_rnd = Rnd(300) + 50                 'Wartezeit: 50 bis 349 ms
            Call Wz(v_rnd)                        'Wartezeit mit Abbruchoption

'011 - Grundhelligkeit mit Zufalls-Blitz
'= = = = = = = = = = = = = = = = = = = =
         Case 11
M011_0:
'Reduzierung der aktuellen Helligkeit auf 20%
            If Pwm_rt > 0 Then Pwm_rt = K_mi + 10
            If Pwm_gn > 0 Then Pwm_gn = K_mi
            If Pwm_bl > 0 Then Pwm_bl = K_mi

'Zuflliges Blitzen
            V_rnd = Rnd(10) + 1                   'Wert: 1 bis 10
            V_rnd = 100 * V_rnd
            Call Wz(v_rnd)                        'Wartezeit mit Abbruchoption

            If Pwm_rt >= K_mi Then
               Pwm_rt = 95
            End If

            If Pwm_gn >= K_mi Then
               Pwm_gn = 95
            End If

            If Pwm_bl >= K_mi Then
               Pwm_bl = 95
            End If
            Call Wz(50)                           'Wartezeit mit Abbruchoption

'Reduzierung der aktuellen Helligkeit auf 20%
            If Pwm_rt > 0 Then Pwm_rt = K_mi + 10 '10: LED-Korrekturwert
            If Pwm_gn > 0 Then Pwm_gn = K_mi
            If Pwm_bl > 0 Then Pwm_bl = K_mi

            If F_st = 1 Then Goto M011_0          'Modus fortfhren

'012 - Wechselnde Helligkeit
'= = = = = = = = = = = = = =
         Case 12
M012_0:
'Reduzierung der aktuellen Helligkeit auf 1%
             If F_dul = 0 Then                    'Durchlauf 1x
               If Pwm_rt > 0 Then Set F_rt
               If Pwm_gn > 0 Then Set F_gn
               If Pwm_bl > 0 Then Set F_bl
               Set F_dul
            End If

'Wechsel der Helligkeit
M012_1:
'Die nderung des Helligkeitswertes erfolgt nur am Ende der PWM-Ansteuerzeit.
            If Z_t0 < 95 Then Goto M012_1

            If F_hew = 0 Then
               Incr Z_hew
               If Z_hew = 90 Then
                  Set F_hew
'Wartezeit Ein
                  Call Wz(1000)
               End If
            Else
               Decr Z_hew
               If Z_hew = 0 Then
                  Reset F_hew
'Wartezeit Aus
                  Call Wz(2000)                   'Wartezeit mit Abbruchoption
               End If
            End If

            If F_rt = 1 Then Pwm_rt = Z_hew
            If F_gn = 1 Then Pwm_gn = Z_hew
            If F_bl = 1 Then Pwm_bl = Z_hew

'Geschwindigkeit fr die nderung der Helligkeit
            Call Wz(50)                           'Wartezeit mit Abbruchoption

         End Select
   End If

M_ende:
'Ende Programmschleife
Loop

'Ende Programm
End

'-------------------------------------------------------------------------------
'30. === I N T E R R U P T R O U T I N E =======================================
'-------------------------------------------------------------------------------
A_310_int_t0:
'
'Timer 0
'-------
Int_tim0:
'Interrupt Timer 0 (8 Bit): 0,05 ms / 20 kHz
'Voreinstellung des Zhlers
   Timer0 = T0_tve                                'Voreinstellung
   Incr Z_t0                                      'Zeitbasis

   If Z_t0 = 100 Then                             'Zhler_T0
      Reset Watchdog                              'Reset Laufzeitberwachung
      Z_t0 = 0
   End If

'*******************************************************************************
'Test - Sequenz/Modus
'Unabhngig von den ber den Bus empfangenen Daten knnen mit dem folgenden Code
'Steuerbefehle simuliert werden.
'>
'  Incr Test_time
'  Select Case Test_time
'- - - - - - - - - - - - - -
'> FT -LED-Farbtest
'    Case 1
'       Twi_db = 250
'    Case 100000
'       Twi_db = 254
'< FT
'- - - - - - - - - - - - - -
'> HW - Helligkeitswechsel
'   Case 1
'      Twi_db = 1
'   Case 100
'      Twi_db = 12
'   Case 200
'      Twi_db = 253
'   Case 390000
'      Twi_db = 254
'< HW
'- - - - - - - - - - - - - -
'  End Select

'  If Test_time >= 121000 Then
'      Test_time = 0
'  End If

'Sequenz (Beispiel)              Sequenz - S-Stop
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'LED-Farbtest                    250     - 254
'
'Modus (Beispiel)                Farbe - Modus - M-Start - M-Stop
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Flackern gelb                   -     - 10    - 253     - 254
'Blitzlicht rot                  1     - 11    - 253     - 254
'Blitzlicht mit Zufallsfarbe:    8     - 11    - 253     - 254
'Wechselnde Helligkeit rot       1     - 12    - 253     - 254
'<
'*******************************************************************************

'PWM-Steuerung der LED (Helligkeit)
'----------------------------------
'Eine lngere Einschaltzeit bedeutet eine hhere Spannung und damit einen
'hhere Helligkeit der LED.
   If Z_t0 = 0 And F_led = 1 Then                 'Zeit-Zhler: 0, LED ein
      If Pwm_rt > 0 Then Set Led_rt
      If Pwm_gn > 0 Then Set Led_gn
      If Pwm_bl > 0 Then Set Led_bl
   End If

'LED ausschalten, wenn die PWM-Leuchtdauerzeit erreicht ist.
   If Z_t0 = Pwm_rt Then Reset Led_rt             'Reset LED_rot
   If Z_t0 = Pwm_gn Then Reset Led_gn             'Reset LED_grn
   If Z_t0 = Pwm_bl Then Reset Led_bl             'Reset LED_blau

Return

'-------------------------------------------------------------------------------
'40. === U N T E R P R O G R A M M =============================================
'-------------------------------------------------------------------------------

'-------------------------------------------------------------------------------
'50. === U N T E R P R O G R A M M - C A L L ===================================
'-------------------------------------------------------------------------------
A_511_callled:
'
'LED-Farbe whlen
'----------------
Sub Led_col(v_rgb)
'Flages Farbe rcksetzen (Modus 12)
   Reset F_rt : Reset F_gn : Reset F_bl

'Zufallsfarbe
   If V_rgb = 8 Then
      V_rgb = Rnd(7) + 1
   End If

'Zuordnung des Datenfeldes (PWM-Werte)
   Select Case V_rgb
      Case 1
         Restore Dat_rt
      Case 2
         Restore Dat_gn
      Case 3
         Restore Dat_bl
      Case 4
         Restore Dat_ge
      Case 5
         Restore Dat_cy
      Case 6
         Restore Dat_ma
      Case 7
         Restore Dat_ws
   End Select

   Read Pwm_rt : Read Pwm_gn : Read Pwm_bl        'Werte aus dem Datenfeld

End Sub

A_512_callwz:
'
'Wartezeit mit Modusabbruch bei Datenempfang
'-------------------------------------------
Sub Wz(v_wz):
   If F_wz = 1 Then Exit Sub                      'bei Wartezeit: Aus
   Z_wz = 0                                       'Zeit-Zhler initialisieren

   While Z_wz < V_wz
      Incr Z_wz
      Waitms 1
      If Twi_db = 254 Then                        'Modus beenden
          Reset F_st                              'muss hier bleiben!!!
          Set F_wz                                'Reset bei Codeauswahl
          Exit Sub
      End If
   Wend

M_wz:
End Sub

'TWI-Bus
'-------
'Ein Master kann Bytes senden oder empfangen.
'Wenn eine Master mehrere Bytes sendet, knnen mehrere Bytes empfangen werden.
'Die folgenden Aufrufe werden durch die USI-Bibliothek aufgerufen.
'Alle Aufrufe sind einzutragen, auch wenn kein Code in den Routinen abzuarbeiten
'ist.

'Aufruf 1
'Der Master sendet "Stop" oder wiederholt "Start"
Twi_stop_rstart_received:
'Programmcode
'
Return

'Aufruf 2
'Der Slave ist adressiert und der Master mchte Daten senden.
Twi_addressed_goread:
'Programmcode
'
'/// Achtung - nicht bei allen Bibliotheken verfgbar ///
'Optional kann ein "NACK" zum Rcksenden der Variable <Twi_ack> gesendet werden.
'Twi_ack = 0
'
Return

'Aufruf 3
'Der Master mchte Daten empfangen und bentigt ein Byte vom Slave
'Anzahl der Bytes in der Variable <Btr>
Twi_addressed_gowrite:
'Programmcode
'
'Anzahl ...... : <Twi_btr>    z. B. Print Twi_btr (Index anzeigen)
'Datenbyte ... : <Twi>        z. B. Twi = 65
'
'/// Achtung - nicht bei allen Bibliotheken verfgbar ///
'Optional kann ein "NACK" zum Rcksenden der Variable <Twi_ack> gesendet werden.
'Twi_ack = 0
'
'Twi = Twi_db
'
Return

'Aufruf 4
'Zuweisung der empfangenen Daten
Twi_gotdata:
  Select Case Twi_btw
'Datenbyte 1
'Es wird vom Master nur ein Datenbyte pro Sendezyklus bertragen.
      Case 1
         Twi_db = Twi
  End Select

Return

'Die Variable <Twi_btr> kann verwendet werden, wenn mehrere Bytes an den Master
'gesendet werden sollen.
Twi_master_needs_byte:
'Programmcode
'
Return

'-------------------------------------------------------------------------------
'70. === F U N K T I O N =======================================================
'-------------------------------------------------------------------------------

'-------------------------------------------------------------------------------
'80. === D A T E N B E R E I C H ===============================================
'-------------------------------------------------------------------------------
A_800_daten:
'
'RGB-Kodierung fr die einzelnen Farben
Dat_rt:                                           'rot
   Data 95 , 0 , 0
Dat_gn:                                           'grn
   Data 0 , 95 , 0
Dat_bl:                                           'blau
   Data 0 , 0 , 95
Dat_ge:                                           'gelb
   Data 95 , 95 , 0
Dat_cy:                                           'cyan
   Data 95 , 0 , 95
Dat_ma:                                           'magenta
   Data 0 , 95 , 95
Dat_ws:                                           'wei
   Data 95 , 95 , 95

'Ende Datenbereich

'###############################################################################
